@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -51,10 +51,11 @@ import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
 /**
- * A MetaClass that implements GroovyObject and behaves like an Expando, allowing the addition of new methods on the fly.
+ * ExpandoMetaClass is a MetaClass that behaves like an Expando, allowing the addition or replacement
+ * of methods, properties and constructors on the fly.
  *
  * Some examples of usage:
- * <code><pre>
+ * <pre>
  * // defines or replaces instance method:
  * metaClass.myMethod = { args -> }
  *
@@ -78,20 +79,171 @@ import java.util.concurrent.ConcurrentHashMap;
  *
  * // defines a new property with an initial value of "blah"
  * metaClass.myProperty = "blah"
- * </code></pre>
+ * </pre>
  *
- * By default methods are only allowed to be added before initialize() is called. In other words you create a new
- * ExpandoMetaClass, add some methods and then call initialize(). If you attempt to add new methods after initialize()
- * has been called an error will be thrown.This is to ensure that the MetaClass can operate appropriately in multi 
- * threaded environments as it forces you to do all method additions at the beginning, before using the MetaClass.
+ * ExpandoMetaClass also supports a DSL/builder like notation to combine multiple definitions together. So instead of this:
+ * <pre>
+ * Number.metaClass.multiply = { Amount amount -> amount.times(delegate) }
+ * Number.metaClass.div =      { Amount amount -> amount.inverse().times(delegate) }
+ * </pre>
+ * You can also now do this:
+ * <pre>
+ * Number.metaClass {
+ *     multiply { Amount amount -> amount.times(delegate) }
+ *     div      { Amount amount -> amount.inverse().times(delegate) }
+ * }
+ * </pre>
  * 
- * ExpandoMetaClass differs here from the default in that it allows you adding methods after initialize has been called.
+ * ExpandoMetaClass also supports runtime mixins. While {@code @Mixin} allows you to mix in new behavior
+ * to classes you own and are designing, you can not easily mixin anything to types you didn't own, e.g.
+ * from third party libraries or from JDK library classes.
+ * Runtime mixins let you add a mixin on any type at runtime.
+ * <pre>
+ * interface Vehicle {
+ *     String getName()
+ * }
+ *
+ * // Category annotation style
+ * {@code @Category}(Vehicle) class FlyingAbility {
+ *     def fly() { "I'm the ${name} and I fly!" }
+ * }
+ *
+ * // traditional category style
+ * class DivingAbility {
+ *     static dive(Vehicle self) { "I'm the ${self.name} and I dive!" }
+ * }
+ *
+ * // provided by a third-party, so can't augment using Mixin annotation
+ * class JamesBondVehicle implements Vehicle {
+ *     String getName() { "James Bond's vehicle" }
+ * }
+ *
+ * // Can be added via metaClass, e.g.:
+ * // JamesBondVehicle.metaClass.mixin DivingAbility, FlyingAbility
+ * // Or using shorthand through DGM method on Class
+ * JamesBondVehicle.mixin DivingAbility, FlyingAbility
+ *
+ * assert new JamesBondVehicle().fly() ==
+ *        "I'm the James Bond's vehicle and I fly!"
+ * assert new JamesBondVehicle().dive() ==
+ *        "I'm the James Bond's vehicle and I dive!"
+ * </pre>
+ * As another example, consider the following class definitions:
+ * <pre>
+ * class Student {
+ *     List<String> schedule = []
+ *     def addLecture(String lecture) { schedule << lecture }
+ * }
+ *
+ * class Worker {
+ *     List<String> schedule = []
+ *     def addMeeting(String meeting) { schedule << meeting }
+ * }
+ * </pre>
+ * We can mimic a form of multiple inheritance as follows:
+ * <pre>
+ * class CollegeStudent {
+ *     static { mixin Student, Worker }
+ * }
+ * new CollegeStudent().with {
+ *     addMeeting('Performance review with Boss')
+ *     addLecture('Learn about Groovy Mixins')
+ *     println schedule
+ *     println mixedIn[Student].schedule
+ *     println mixedIn[Worker].schedule
+ * }
+ * </pre>
+ * Which outputs these lines when run:
+ * <pre>
+ * [Performance review with Boss]
+ * [Learn about Groovy Mixins]
+ * [Performance review with Boss]
+ * </pre>
+ * Perhaps some explanation is required here. The methods and properties of Student and Worker are
+ * added to CollegeStudent. Worker is added last, so for overlapping methods, its methods will
+ * be used, e.g. when calling <code>schedule</code>, it will be the schedule property (getSchedule method)
+ * from Worker that is used. The schedule property from Student will be shadowed but the <code>mixedIn</code>
+ * notation allows us to get to that too if we need as the last two lines show.
+ * <p/>
+ * We can also be a little more dynamic and not require the CollegeStudent class to
+ * be defined at all, e.g.:
+ * <pre>
+ * def cs = new Object()
+ * cs.metaClass {
+ *     mixin Student, Worker
+ *     getSchedule {
+ *         mixedIn[Student].schedule + mixedIn[Worker].schedule
+ *     }
+ * }
+ * cs.with {
+ *     addMeeting('Performance review with Boss')
+ *     addLecture('Learn about Groovy Mixins')
+ *     println schedule
+ * }
+ * </pre>
+ * Which outputs this line when run:
+ * <pre>
+ * [Learn about Groovy Mixins, Performance review with Boss]
+ * </pre>
+ * As another example, we can also define a no dup queue by mixing in some
+ * Queue and Set functionality as follows:
+ * <pre>
+ * def ndq = new Object()
+ * ndq.metaClass {
+ *     mixin ArrayDeque
+ *     mixin HashSet
+ *     leftShift = { Object o ->
+ *         if (!mixedIn[Set].contains(o)) {
+ *             mixedIn[Queue].push(o)
+ *             mixedIn[Set].add(o)
+ *         }
+ *     }
+ * }
+ * ndq << 1
+ * ndq << 2
+ * ndq << 1
+ * assert ndq.size() == 2
+ * </pre>
+ * As a final example, we sometimes need to pass such mixed in classes or objects
+ * into Java methods which require a given static type but the ExpandoMetaClass mixin approach uses a very dynamic
+ * approach based on duck typing rather than static interface definitions, so doesn't by default
+ * produce objects matching the required static type. Luckily, there is a mixins capability
+ * within ExpandoMetaClass which supports the use of Groovy's common 'as StaticType' notation to produce an object
+ * having the correct static type so that it can be passed to the Java method call in question.
+ * A slightly contrived example illustrating this feature:
+ * <pre>
+ * class CustomComparator implements Comparator {
+ *     int compare(Object a, b) { return a.size() - b.size() }
+ * }
+ *
+ * class CustomCloseable implements Closeable {
+ *     void close() { println 'Lights out - I am closing' }
+ * }
+ *
+ * import static mypackage.IOUtils.closeQuietly
+ * import static java.util.Collections.sort
+ * def o = new Object()
+ * o.metaClass.mixin CustomComparator, CustomCloseable
+ * def items = ['a', 'bbb', 'cc']
+ * sort(items, o as Comparator)
+ * println items                // => [a, cc, bbb]
+ * closeQuietly(o as Closeable) // => Lights out - I am closing
+ * </pre>
+ *
+ * <b>Further details</b>
+ * <p/>
+ * When using the default implementations of MetaClass, methods are only allowed to be added before initialize() is called.
+ * In other words you create a new MetaClass, add some methods and then call initialize(). If you attempt to add new methods
+ * after initialize() has been called, an error will be thrown. This is to ensure that the MetaClass can operate appropriately
+ * in multi-threaded environments as it forces you to do all method additions at the beginning, before using the MetaClass.
+ * <p/>
+ * ExpandoMetaClass differs here from the default in that it allows you to add methods after initialize has been called.
  * This is done by setting the initialize flag internally to false and then add the methods. Since this is not thread 
  * safe it has to be done in a synchronized block. The methods to check for modification and initialization are
  * therefore synchronized as well. Any method call done through this meta class will first check if the it is 
  * synchronized. Should this happen during a modification, then the method cannot be selected or called unless the 
  * modification is completed. 
- *
+ * <p/>
  * WARNING: This MetaClass uses a thread-bound ThreadLocal instance to store and retrieve properties.
  * In addition properties stored use soft references so they are both bound by the life of the Thread and by the soft
  * references. The implication here is you should NEVER use dynamic properties if you want their values to stick around
@@ -223,7 +375,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
     }
 
     protected void onSuperMethodFoundInHierarchy(MetaMethod method) {
-        addSuperMethodIfNotOverriden(method);
+        addSuperMethodIfNotOverridden(method);
     }
 
     protected void onSuperPropertyFoundInHierarchy(MetaBeanProperty property) {
@@ -327,7 +479,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
         this.initialized = b;
     }
 
-    private void addSuperMethodIfNotOverriden(final MetaMethod metaMethodFromSuper) {
+    private void addSuperMethodIfNotOverridden(final MetaMethod metaMethodFromSuper) {
 		performOperationOnMetaClass(new Callable() {
 			public void call() {
 
@@ -873,7 +1025,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
                     continue; // don't inherit static methods except our own
                 registerStaticMethod(metaMethod.getName(), (Closure) ((ClosureStaticMetaMethod) metaMethod).getClosure().clone());
             } else
-                addSuperMethodIfNotOverriden(metaMethod);
+                addSuperMethodIfNotOverridden(metaMethod);
         }
         Collection<MetaProperty> metaProperties = superExpando.getExpandoProperties();
         for (Object metaProperty : metaProperties) {
@@ -904,7 +1056,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
     }
 
     /**
-     * Overrides default implementation just in case invokeMethod has been overriden by ExpandoMetaClass
+     * Overrides default implementation just in case invokeMethod has been overridden by ExpandoMetaClass
      *
      * @see groovy.lang.MetaClassImpl#invokeMethod(Class, Object, String, Object[], boolean, boolean)
      */
@@ -944,7 +1096,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
     }
 
     /**
-     * Overrides default implementation just in case getProperty method has been overriden by ExpandoMetaClass
+     * Overrides default implementation just in case getProperty method has been overridden by ExpandoMetaClass
      *
      * @see MetaClassImpl#getProperty(Object, String)
      */
@@ -960,7 +1112,7 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
     }
 
     /**
-     * Overrides default implementation just in case setProperty method has been overriden by ExpandoMetaClass
+     * Overrides default implementation just in case setProperty method has been overridden by ExpandoMetaClass
      *
      * @see MetaClassImpl#setProperty(Class, Object, String, Object, boolean, boolean)
      */
@@ -1121,7 +1273,9 @@ public class ExpandoMetaClass extends MetaClassImpl implements GroovyObject {
         Class[] params = MetaClassHelper.convertToTypeArray(args);
         MetaMethod method = pickMethod(GROOVY_CONSTRUCTOR, params);
         if(method!=null && method.getParameterTypes().length == args.length) {
-           return new ConstructorMetaMethodSite(site, this, method, params);
+        	if(method.getDeclaringClass().getTheClass().equals(getTheClass())) {
+                return new ConstructorMetaMethodSite(site, this, method, params);
+        	}
         }
 
         return super.createConstructorSite(site, args);
