@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@ import org.codehaus.groovy.ast.ClassNode;
 import org.codehaus.groovy.ast.ModuleNode;
 import org.codehaus.groovy.classgen.Verifier;
 import org.codehaus.groovy.control.*;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 import org.objectweb.asm.ClassVisitor;
 import org.objectweb.asm.ClassWriter;
 
@@ -37,10 +38,10 @@ import java.util.*;
 
 /**
  * A ClassLoader which can load Groovy classes. The loaded classes are cached,
- * classes from other classlaoders should not be cached. To be able to load a
+ * classes from other classloaders should not be cached. To be able to load a
  * script that was asked for earlier but was created later it is essential not
  * to keep anything like a "class not found" information for that class name.
- * This includes possible parent loaders. Classes that are not chached are always
+ * This includes possible parent loaders. Classes that are not cached are always
  * reloaded.
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
@@ -49,7 +50,7 @@ import java.util.*;
  * @author Bing Ran
  * @author <a href="mailto:scottstirling@rcn.com">Scott Stirling</a>
  * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
- * @version $Revision: 16982 $
+ * @version $Revision: 19648 $
  */
 public class GroovyClassLoader extends URLClassLoader {
 
@@ -102,7 +103,7 @@ public class GroovyClassLoader extends URLClassLoader {
     }
 
     /**
-     * creates a GroovyClassLaoder.
+     * creates a GroovyClassLoader.
      *
      * @param parent                    the parent class loader
      * @param config                    the compiler configuration
@@ -186,7 +187,7 @@ public class GroovyClassLoader extends URLClassLoader {
      * @return the main class defined in the given script
      */
     public Class parseClass(File file) throws CompilationFailedException, IOException {
-        return parseClass(new GroovyCodeSource(file));
+        return parseClass(new GroovyCodeSource(file, config.getSourceEncoding()));
     }
 
     /**
@@ -196,14 +197,14 @@ public class GroovyClassLoader extends URLClassLoader {
      * @param fileName the file name to use as the name of the class
      * @return the main class defined in the given script
      */
-    public Class parseClass(String text, String fileName) throws CompilationFailedException {
-        byte[] bytes = null;
-        try {
-            bytes = text.getBytes(config.getSourceEncoding());
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(1,null,e);
-        }
-        return parseClass(new ByteArrayInputStream(bytes), fileName);
+    public Class parseClass(final String text, final String fileName) throws CompilationFailedException {
+        GroovyCodeSource gcs = (GroovyCodeSource) AccessController.doPrivileged(new PrivilegedAction() {
+            public Object run() {
+                return new GroovyCodeSource(text, fileName, "/groovy/script");
+            }
+        });
+        gcs.setCachable(false);
+        return parseClass(gcs);
     }
 
     /**
@@ -213,7 +214,8 @@ public class GroovyClassLoader extends URLClassLoader {
      * @return the main class defined in the given script
      */
     public Class parseClass(String text) throws CompilationFailedException {
-        return parseClass(text, "script" + System.currentTimeMillis() + ".groovy");
+        return parseClass(text, "script" + System.currentTimeMillis() + 
+                Math.abs(text.hashCode()) + ".groovy");
     }
 
     /**
@@ -221,6 +223,8 @@ public class GroovyClassLoader extends URLClassLoader {
      *
      * @param in an InputStream
      * @return the main class defined in the given script
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
      */
     public Class parseClass(InputStream in) throws CompilationFailedException {
         return parseClass(in, generateScriptName());
@@ -231,6 +235,9 @@ public class GroovyClassLoader extends URLClassLoader {
         return "script" + scriptNameCounter + ".groovy";
     }
 
+    /**
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
+     */
     public Class parseClass(final InputStream in, final String fileName) throws CompilationFailedException {
         // For generic input streams, provide a catch-all codebase of
         // GroovyScript
@@ -238,7 +245,14 @@ public class GroovyClassLoader extends URLClassLoader {
         // a codebase of file:groovy.script
         GroovyCodeSource gcs = (GroovyCodeSource) AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
-                return new GroovyCodeSource(in, fileName, "/groovy/script");
+                try {
+                    String scriptText = config.getSourceEncoding() != null ?
+                            DefaultGroovyMethods.getText(in, config.getSourceEncoding()) :
+                            DefaultGroovyMethods.getText(in);
+                    return new GroovyCodeSource(scriptText, fileName, "/groovy/script");
+                } catch (IOException e) {
+                    throw new RuntimeException("Impossible to read the content of the input stream for file named: " + fileName, e);
+                }
             }
         });
         return parseClass(gcs);
@@ -266,7 +280,7 @@ public class GroovyClassLoader extends URLClassLoader {
             CompilationUnit unit = createCompilationUnit(config, codeSource.getCodeSource());
             SourceUnit su = null;
             if (codeSource.getFile() == null) {
-                su = unit.addSource(codeSource.getName(), codeSource.getInputStream());
+                su = unit.addSource(codeSource.getName(), codeSource.getScriptText());
             } else {
                 su = unit.addSource(codeSource.getFile());
             }
@@ -447,7 +461,7 @@ public class GroovyClassLoader extends URLClassLoader {
      * creates a ClassCollector for a new compilation.
      *
      * @param unit the compilationUnit
-     * @param su   the SoruceUnit
+     * @param su   the SourceUnit
      * @return the ClassCollector
      */
     protected ClassCollector createCollector(CompilationUnit unit, SourceUnit su) {
@@ -583,11 +597,11 @@ public class GroovyClassLoader extends URLClassLoader {
     }
 
     /**
-     * Indicates if a class is recompilable. Recompileable means, that the classloader
+     * Indicates if a class is recompilable. Recompilable means, that the classloader
      * will try to locate a groovy source file for this class and then compile it again,
      * adding the resulting class as entry to the cache. Giving null as class is like a
      * recompilation, so the method should always return true here. Only classes that are
-     * implementing GroovyObject are compileable and only if the timestamp in the class
+     * implementing GroovyObject are compilable and only if the timestamp in the class
      * is lower than Long.MAX_VALUE.
      * <p/>
      * NOTE: First the parent loaders will be asked and only if they don't return a
@@ -787,7 +801,7 @@ public class GroovyClassLoader extends URLClassLoader {
         try {
             decodedFile = URLDecoder.decode(fileName, "UTF-8");
         } catch (UnsupportedEncodingException e) {
-            System.err.println("Encounted an invalid encoding scheme when trying to use URLDecoder.decode() inside of the GroovyClassLoader.decodeFileName() method.  Returning the unencoded URL.");
+            System.err.println("Encountered an invalid encoding scheme when trying to use URLDecoder.decode() inside of the GroovyClassLoader.decodeFileName() method.  Returning the unencoded URL.");
             System.err.println("Please note that if you encounter this error and you have spaces in your directory you will run into issues.  Refer to GROOVY-1787 for description of this bug.");
         }
 
