@@ -16,10 +16,12 @@
 package groovy.lang;
 
 import groovy.ui.GroovyMain;
+import groovy.security.GroovyCodeSourcePermission;
 
 import org.codehaus.groovy.control.CompilationFailedException;
 import org.codehaus.groovy.control.CompilerConfiguration;
 import org.codehaus.groovy.runtime.InvokerHelper;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 import java.io.*;
 import java.lang.reflect.Constructor;
@@ -35,13 +37,16 @@ import java.util.Map;
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Guillaume Laforge
- * @version $Revision: 13745 $
+ * @author Paul King
+ * @version $Revision: 18805 $
  */
 public class GroovyShell extends GroovyObjectSupport {
-       
+
+    public static final String DEFAULT_CODE_BASE = "/groovy/shell";
+
+    @Deprecated
     public static final String[] EMPTY_ARGS = {};
 
-    
     private Binding context;
     private int counter;
     private CompilerConfiguration config;
@@ -83,15 +88,16 @@ public class GroovyShell extends GroovyObjectSupport {
             throw new IllegalArgumentException("Compiler configuration must not be null.");
         }
         final ClassLoader parentLoader = (parent!=null)?parent:GroovyShell.class.getClassLoader();
-        this.loader = (GroovyClassLoader) AccessController.doPrivileged(new PrivilegedAction() {
-            public Object run() {
+        this.loader = AccessController.doPrivileged(new PrivilegedAction<GroovyClassLoader>() {
+            public GroovyClassLoader run() {
                 return new GroovyClassLoader(parentLoader,config);
             }
         });
         this.context = binding;        
         this.config = config;
     }
-    
+
+    @Deprecated
     public void initializeBinding() {
         Map map = context.getVariables();
         if (map.get("shell")==null) map.put("shell",this);
@@ -136,6 +142,10 @@ public class GroovyShell extends GroovyObjectSupport {
         }
     }
 
+    //
+    // FIXME: Use List<String> here, current version is not safe
+    //
+
     /**
      * A helper method which runs the given script file with the given command line arguments
      *
@@ -171,7 +181,6 @@ public class GroovyShell extends GroovyObjectSupport {
         int p = scriptName.lastIndexOf(".");
         if (p++ >= 0) {
             if (scriptName.substring(p).equals("java")) {
-                System.err.println("error: cannot compile file with .java extension: " + scriptName);
                 throw new CompilationFailedException(0, null);
             }
         }
@@ -199,8 +208,8 @@ public class GroovyShell extends GroovyObjectSupport {
         // if you are compiling the script because the JVM isn't executing the main method.
         Class scriptClass;
         try {
-            scriptClass = (Class) AccessController.doPrivileged(new PrivilegedExceptionAction() {
-                public Object run() throws CompilationFailedException, IOException {
+            scriptClass = AccessController.doPrivileged(new PrivilegedExceptionAction<Class>() {
+                public Class run() throws CompilationFailedException, IOException {
                     return loader.parseClass(scriptFile);
                 }
             });
@@ -239,23 +248,23 @@ public class GroovyShell extends GroovyObjectSupport {
         if (scriptClass == null) {
             return null;
         }
-        try {
-            if (Script.class.isAssignableFrom(scriptClass)) {
-                // treat it just like a script if it is one
-                Script script  = null;
-                try {
-                    script = (Script) scriptClass.newInstance();
-                } catch (InstantiationException e) {
-                    // ignore instaintiations errors,, try to do main
-                } catch (IllegalAccessException e) {
-                   // ignore instaintiations errors, try to do main
-                }
-                if (script != null) {
-                    script.setBinding(context);
-                    script.setProperty("args", args);
-                    return script.run();
-                }
+        if (Script.class.isAssignableFrom(scriptClass)) {
+            // treat it just like a script if it is one
+            Script script  = null;
+            try {
+                script = (Script) scriptClass.newInstance();
+            } catch (InstantiationException e) {
+                // ignore instaintiations errors,, try to do main
+            } catch (IllegalAccessException e) {
+               // ignore instaintiations errors, try to do main
             }
+            if (script != null) {
+                script.setBinding(context);
+                script.setProperty("args", args);
+                return script.run();
+            }
+        }
+        try {
             // let's find a main method
             scriptClass.getMethod("main", new Class[]{String[].class});
             // if that main method exist, invoke it
@@ -269,6 +278,10 @@ public class GroovyShell extends GroovyObjectSupport {
             if (isJUnit3Test(scriptClass)) {
                 return runJUnit3Test(scriptClass);
             }
+            // if it's a JUnit 3.8.x test suite, run it with an appropriate runner
+            if (isJUnit3TestSuite(scriptClass)) {
+                return runJUnit3TestSuite(scriptClass);
+            }
             // if it's a JUnit 4.x test, run it with an appropriate runner
             if (isJUnit4Test(scriptClass)) {
                 return runJUnit4Test(scriptClass);
@@ -304,7 +317,7 @@ public class GroovyShell extends GroovyObjectSupport {
                 constructor = scriptClass.getConstructor(new Class[]{});
                 try {
                     // instantiate a runnable and run it
-                    runnable = (Runnable) constructor.newInstance(new Object[]{});
+                    runnable = (Runnable) constructor.newInstance();
                 } catch (Throwable t) {
                     reason = t;
                 }
@@ -333,25 +346,42 @@ public class GroovyShell extends GroovyObjectSupport {
             Object testSuite = InvokerHelper.invokeConstructorOf("junit.framework.TestSuite",new Object[]{scriptClass});
             return InvokerHelper.invokeStaticMethod("junit.textui.TestRunner", "run", new Object[]{testSuite});
         } catch (ClassNotFoundException e) {
-            throw new GroovyRuntimeException("Failed to run the unit test. JUnit is not on the Classpath.");
+            throw new GroovyRuntimeException("Failed to run the unit test. JUnit is not on the Classpath.", e);
+        }
+    }
+
+    /**
+     * Run the specified class extending TestSuite as a unit test.
+     * This is done through reflection, to avoid adding a dependency to the JUnit framework.
+     * Otherwise, developers embedding Groovy and using GroovyShell to load/parse/compile
+     * groovy scripts and classes would have to add another dependency on their classpath.
+     *
+     * @param scriptClass the class to be run as a unit test
+     */
+    private Object runJUnit3TestSuite(Class scriptClass) {
+        try {
+            Object testSuite = InvokerHelper.invokeStaticMethod(scriptClass, "suite", new Object[]{});
+            return InvokerHelper.invokeStaticMethod("junit.textui.TestRunner", "run", new Object[]{testSuite});
+        } catch (ClassNotFoundException e) {
+            throw new GroovyRuntimeException("Failed to run the unit test. JUnit is not on the Classpath.", e);
         }
     }
 
     private Object runJUnit4Test(Class scriptClass) {
         try {
             return InvokerHelper.invokeStaticMethod("org.codehaus.groovy.vmplugin.v5.JUnit4Utils",
-                    "realRunJUnit4Test", new Object[]{scriptClass});
+                    "realRunJUnit4Test", new Object[]{scriptClass, this.loader});
         } catch (ClassNotFoundException e) {
-            throw new GroovyRuntimeException("Failed to run the JUnit 4 test.");
+            throw new GroovyRuntimeException("Failed to run the JUnit 4 test.", e);
         }
     }
 
     private Object runTestNgTest(Class scriptClass) {
         try {
             return InvokerHelper.invokeStaticMethod("org.codehaus.groovy.vmplugin.v5.TestNgUtils",
-                    "realRunTestNgTest", new Object[]{scriptClass});
+                    "realRunTestNgTest", new Object[]{scriptClass, this.loader});
         } catch (ClassNotFoundException e) {
-            throw new GroovyRuntimeException("Failed to run the TestNG test.");
+            throw new GroovyRuntimeException("Failed to run the TestNG test.", e);
         }
     }
 
@@ -382,6 +412,33 @@ public class GroovyShell extends GroovyObjectSupport {
         return isUnitTestCase;
     }
 
+     /**
+     * Utility method to check through reflection if the class appears to be a
+     * JUnit 3.8.x test suite, i.e.&nsbp;checks if it extends JUnit 3.8.x's TestSuite.
+     *
+     * @param scriptClass the class we want to check
+     * @return true if the class appears to be a test
+     */
+    private boolean isJUnit3TestSuite(Class scriptClass) {
+        // check if the parsed class is a TestSuite,
+        // so that it is possible to run it as a JUnit test
+        boolean isUnitTestSuite = false;
+        try {
+            try {
+                Class testSuiteClass = this.loader.loadClass("junit.framework.TestSuite");
+                // if scriptClass extends TestSuiteClass
+                if (testSuiteClass.isAssignableFrom(scriptClass)) {
+                    isUnitTestSuite = true;
+                }
+            } catch (ClassNotFoundException e) {
+                // fall through
+            }
+        } catch (Throwable e) {
+            // fall through
+        }
+        return isUnitTestSuite;
+    }
+
     /**
      * Utility method to check via reflection if the parsed class appears to be a JUnit4
      * test, i.e.&nsbp;checks whether it appears to be using the relevant JUnit 4 annotations.
@@ -406,7 +463,7 @@ public class GroovyShell extends GroovyObjectSupport {
                 isTest = true;
             };
         } catch (ClassNotFoundException e) {
-            throw new GroovyRuntimeException("Failed to invoke the JUnit 4 helper class.");
+            throw new GroovyRuntimeException("Failed to invoke the JUnit 4 helper class.", e);
         }
         return isTest;
     }
@@ -434,7 +491,7 @@ public class GroovyShell extends GroovyObjectSupport {
                 isTest = true;
             };
         } catch (ClassNotFoundException e) {
-            throw new GroovyRuntimeException("Failed to invoke the TestNG helper class.");
+            throw new GroovyRuntimeException("Failed to invoke the TestNG helper class.", e);
         }
         return isTest;
     }
@@ -446,12 +503,14 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param fileName   is the logical file name of the script (which is used to create the class name of the script)
      * @param args       the command line arguments to pass in
      */
-    public Object run(String scriptText, String fileName, String[] args) throws CompilationFailedException {
-        try {
-            return run(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), fileName, args);
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
-        }
+    public Object run(final String scriptText, final String fileName, String[] args) throws CompilationFailedException {
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+            public GroovyCodeSource run() {
+                return new GroovyCodeSource(scriptText, fileName, DEFAULT_CODE_BASE);
+            }
+        });
+        Class scriptClass = parseClass(gcs);
+        return runScriptOrMainOrTestOrRunnable(scriptClass, args);
     }
 
     /**
@@ -461,10 +520,36 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param fileName is the logical file name of the script (which is used to create the class name of the script)
      * @param args     the command line arguments to pass in
      */
+    public Object run(final Reader in, final String fileName, String[] args) throws CompilationFailedException {
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+                    public GroovyCodeSource run() {
+                        return new GroovyCodeSource(in, fileName, DEFAULT_CODE_BASE);
+                    }
+        });
+        Class scriptClass = parseClass(gcs);
+        return runScriptOrMainOrTestOrRunnable(scriptClass, args);
+    }
+
+    /**
+     * Runs the given script with command line arguments
+     *
+     * @param in       the stream reading the script
+     * @param fileName is the logical file name of the script (which is used to create the class name of the script)
+     * @param args     the command line arguments to pass in
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
+     */
     public Object run(final InputStream in, final String fileName, String[] args) throws CompilationFailedException {
-        GroovyCodeSource gcs = (GroovyCodeSource) AccessController.doPrivileged(new PrivilegedAction() {
-            public Object run() {
-                return new GroovyCodeSource(in, fileName, "/groovy/shell");
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+            public GroovyCodeSource run() {
+                try {
+                    String scriptText = config.getSourceEncoding() != null ?
+                            DefaultGroovyMethods.getText(in, config.getSourceEncoding()) :
+                            DefaultGroovyMethods.getText(in);
+                    return new GroovyCodeSource(scriptText, fileName, DEFAULT_CODE_BASE);
+                } catch (IOException e) {
+                    throw new RuntimeException("Impossible to read the content of the input stream for file named: " + fileName, e);
+                }
             }
         });
         Class scriptClass = parseClass(gcs);
@@ -484,13 +569,22 @@ public class GroovyShell extends GroovyObjectSupport {
      *
      * @param codeSource
      * @throws CompilationFailedException
-     * @throws CompilationFailedException
      */
     public Object evaluate(GroovyCodeSource codeSource) throws CompilationFailedException {
         Script script = parse(codeSource);
+        script.setBinding(context);
         return script.run();
     }
 
+    /**
+     * Evaluates some script against the current Binding and returns the result
+     *
+     * @param scriptText the text of the script
+     */
+    public Object evaluate(final String scriptText) throws CompilationFailedException {
+        return evaluate(scriptText, generateScriptName(), DEFAULT_CODE_BASE);
+    }
+
     /**
      * Evaluates some script against the current Binding and returns the result
      *
@@ -498,23 +592,26 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param fileName   is the logical file name of the script (which is used to create the class name of the script)
      */
     public Object evaluate(String scriptText, String fileName) throws CompilationFailedException {
-        try {
-            return evaluate(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), fileName);
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
-        }
+        return evaluate(scriptText, fileName, DEFAULT_CODE_BASE);
     }
 
     /**
      * Evaluates some script against the current Binding and returns the result.
      * The .class file created from the script is given the supplied codeBase
      */
-    public Object evaluate(String scriptText, String fileName, String codeBase) throws CompilationFailedException {
-        try {
-            return evaluate(new GroovyCodeSource(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), fileName, codeBase));
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
-        }
+    public Object evaluate(final String scriptText, final String fileName, final String codeBase) throws CompilationFailedException {
+        SecurityManager sm = System.getSecurityManager();
+		if (sm != null) {
+		    sm.checkPermission(new GroovyCodeSourcePermission(codeBase));
+		}
+
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+            public GroovyCodeSource run() {
+                return new GroovyCodeSource(scriptText, fileName, codeBase);
+            }
+        });
+
+        return evaluate(gcs);
     }
 
     /**
@@ -523,19 +620,34 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param file is the file of the script (which is used to create the class name of the script)
      */
     public Object evaluate(File file) throws CompilationFailedException, IOException {
-        return evaluate(new GroovyCodeSource(file));
+        return evaluate(new GroovyCodeSource(file, config.getSourceEncoding()));
     }
 
     /**
      * Evaluates some script against the current Binding and returns the result
      *
-     * @param scriptText the text of the script
+     * @param in the stream reading the script
      */
-    public Object evaluate(String scriptText) throws CompilationFailedException {
+    public Object evaluate(Reader in) throws CompilationFailedException {
+        return evaluate(in, generateScriptName());
+    }
+
+    /**
+     * Evaluates some script against the current Binding and returns the result
+     *
+     * @param in       the stream reading the script
+     * @param fileName is the logical file name of the script (which is used to create the class name of the script)
+     */
+    public Object evaluate(Reader in, String fileName) throws CompilationFailedException {
+        Script script = null;
         try {
-            return evaluate(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), generateScriptName());
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
+            script = parse(in, fileName);
+            script.setBinding(context);
+            return script.run();
+        } finally {
+            if (script != null) {
+                InvokerHelper.removeClass(script.getClass());
+            }
         }
     }
 
@@ -543,6 +655,8 @@ public class GroovyShell extends GroovyObjectSupport {
      * Evaluates some script against the current Binding and returns the result
      *
      * @param in the stream reading the script
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
      */
     public Object evaluate(InputStream in) throws CompilationFailedException {
         return evaluate(in, generateScriptName());
@@ -553,11 +667,14 @@ public class GroovyShell extends GroovyObjectSupport {
      *
      * @param in       the stream reading the script
      * @param fileName is the logical file name of the script (which is used to create the class name of the script)
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
      */
     public Object evaluate(InputStream in, String fileName) throws CompilationFailedException {
         Script script = null;
         try {
             script = parse(in, fileName);
+            script.setBinding(context);
             return script.run();
         } finally {
             if (script != null) {
@@ -566,17 +683,37 @@ public class GroovyShell extends GroovyObjectSupport {
         }
     }
 
+    /**
+     * Parses the given script and returns it ready to be run
+     *
+     * @param reader    the stream reading the script
+     * @param fileName  is the logical file name of the script (which is used to create the class name of the script)
+     * @return the parsed script which is ready to be run via @link Script.run()
+     */
+    public Script parse(final Reader reader, final String fileName) throws CompilationFailedException {
+        return parse(new GroovyCodeSource(reader, fileName, DEFAULT_CODE_BASE));
+    }
+
     /**
      * Parses the given script and returns it ready to be run
      *
      * @param in       the stream reading the script
      * @param fileName is the logical file name of the script (which is used to create the class name of the script)
      * @return the parsed script which is ready to be run via @link Script.run()
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
      */
     public Script parse(final InputStream in, final String fileName) throws CompilationFailedException {
-        GroovyCodeSource gcs = (GroovyCodeSource) AccessController.doPrivileged(new PrivilegedAction() {
-            public Object run() {
-                return new GroovyCodeSource(in, fileName, "/groovy/shell");
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+            public GroovyCodeSource run() {
+                try {
+                    String scriptText = config.getSourceEncoding() != null ?
+                            DefaultGroovyMethods.getText(in, config.getSourceEncoding()) :
+                            DefaultGroovyMethods.getText(in);
+                    return new GroovyCodeSource(scriptText, fileName, DEFAULT_CODE_BASE);
+                } catch (IOException e) {
+                    throw new RuntimeException("Impossible to read the content of the input stream for file named: " + fileName, e);
+                }
             }
         });
         return parse(gcs);
@@ -608,7 +745,7 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param file is the file of the script (which is used to create the class name of the script)
      */
     public Script parse(File file) throws CompilationFailedException, IOException {
-        return parse(new GroovyCodeSource(file));
+        return parse(new GroovyCodeSource(file, config.getSourceEncoding()));
     }
 
     /**
@@ -617,19 +754,16 @@ public class GroovyShell extends GroovyObjectSupport {
      * @param scriptText the text of the script
      */
     public Script parse(String scriptText) throws CompilationFailedException {
-        try {
-            return parse(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), generateScriptName());
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
-        }
+        return parse(scriptText, generateScriptName());
     }
 
-    public Script parse(String scriptText, String fileName) throws CompilationFailedException {
-        try {
-            return parse(new ByteArrayInputStream(scriptText.getBytes(config.getSourceEncoding())), fileName);
-        } catch (UnsupportedEncodingException e) {
-            throw new CompilationFailedException(0, null, e);
-        }
+    public Script parse(final String scriptText, final String fileName) throws CompilationFailedException {
+        GroovyCodeSource gcs = AccessController.doPrivileged(new PrivilegedAction<GroovyCodeSource>() {
+            public GroovyCodeSource run() {
+                return new GroovyCodeSource(scriptText, fileName, DEFAULT_CODE_BASE);
+            }
+        });
+        return parse(gcs);
     }
 
     /**
@@ -637,6 +771,17 @@ public class GroovyShell extends GroovyObjectSupport {
      *
      * @param in the stream reading the script
      */
+    public Script parse(Reader in) throws CompilationFailedException {
+        return parse(in, generateScriptName());
+    }
+
+    /**
+     * Parses the given script and returns it ready to be run
+     *
+     * @param in the stream reading the script
+     *
+     * @deprecated Prefer using methods taking a Reader rather than an InputStream to avoid wrong encoding issues.
+     */
     public Script parse(InputStream in) throws CompilationFailedException {
         return parse(in, generateScriptName());
     }
