@@ -83,13 +83,24 @@ import javax.servlet.http.HttpServletResponse;
  *   template.engine = groovy.text.XmlTemplateEngine
  * </pre>
  * 
+ * <h3>Servlet Init Parameters</h3>
+ * 
  * <h4>Logging and extra-output options</h4>
  *
  * <p>
  * This implementation provides a verbosity flag switching log statements.
  * The servlet init parameter name is:
  * <pre>
- *   generate.by = true(default) | false
+ *   generated.by = true(default) | false
+ * </pre>
+ *
+ * <h4>Groovy Source Encoding Parameter</h4>
+ *
+ * <p>
+ * The following servlet init parameter name can be used to specify the encoding TemplateServlet will use
+ * to read the template groovy source files:
+ * <pre>
+ *   groovy.source.encoding
  * </pre>
  * 
  * @see TemplateServlet#setVariables(ServletBinding)
@@ -181,6 +192,10 @@ public class TemplateServlet extends AbstractHttpServlet {
      */
     private boolean generateBy;
 
+    private String fileEncodingParamVal;
+    
+    private static final String GROOVY_SOURCE_ENCODING = "groovy.source.encoding";
+
     /**
      * Create new TemplateSerlvet.
      */
@@ -188,6 +203,7 @@ public class TemplateServlet extends AbstractHttpServlet {
         this.cache = new WeakHashMap();
         this.engine = null; // assigned later by init()
         this.generateBy = true; // may be changed by init()
+        this.fileEncodingParamVal = null; // may be changed by init()
     }
 
     /**
@@ -246,7 +262,8 @@ public class TemplateServlet extends AbstractHttpServlet {
                 log("Creating new template from file " + file + "...");
             }
             
-            String fileEncoding = System.getProperty("groovy.source.encoding");
+            String fileEncoding = (fileEncodingParamVal != null) ? fileEncodingParamVal : 
+            	System.getProperty(GROOVY_SOURCE_ENCODING);
             
             Reader reader = null;
             try {
@@ -306,6 +323,10 @@ public class TemplateServlet extends AbstractHttpServlet {
         if (value != null) {
             this.generateBy = Boolean.valueOf(value).booleanValue();
         }
+        value = config.getInitParameter(GROOVY_SOURCE_ENCODING);
+        if (value != null) {
+            this.fileEncodingParamVal = value;
+        }
         log("Servlet " + getClass().getName() + " initialized on " + engine.getClass());
     }
 
