@@ -16,6 +16,7 @@
 package groovy.ui;
 
 import groovy.lang.GroovyShell;
+import groovy.lang.GroovySystem;
 import groovy.lang.Script;
 
 import java.io.*;
@@ -32,15 +33,15 @@ import org.apache.commons.cli.ParseException;
 import org.apache.commons.cli.PosixParser;
 import org.codehaus.groovy.control.CompilationFailedException;
 import org.codehaus.groovy.control.CompilerConfiguration;
-import org.codehaus.groovy.runtime.InvokerHelper;
 import org.codehaus.groovy.runtime.InvokerInvocationException;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 /**
  * A Command line to execute groovy.
  *
  * @author Jeremy Rayner
  * @author Yuri Schimke
- * @version $Revision: 12135 $
+ * @version $Revision: 19532 $
  */
 public class GroovyMain {
 
@@ -103,7 +104,7 @@ public class GroovyMain {
             if (cmd.hasOption('h')) {
                 printHelp(out, options);
             } else if (cmd.hasOption('v')) {
-                String version = InvokerHelper.getVersion();
+                String version = GroovySystem.getVersion();
                 out.println("Groovy Version: " + version + " JVM: " + System.getProperty("java.version"));
             } else {
                 // If we fail, then exit with an error so scripting frameworks can catch it
@@ -206,7 +207,7 @@ public class GroovyMain {
         options.addOption(
             OptionBuilder.withArgName("port")
             .hasOptionalArg()
-            .withDescription("listen on a port and process inbound lines")
+            .withDescription("listen on a port and process inbound lines (default: 1960)")
             .create('l'));
         options.addOption(
             OptionBuilder.withArgName("splitPattern")
@@ -340,7 +341,7 @@ public class GroovyMain {
         GroovyShell groovy = new GroovyShell(conf);
         //check the script is currently valid before starting a server against the script
         if (isScriptFile) {
-            groovy.parse(new FileInputStream(huntForTheScriptFile(script)));
+            groovy.parse(DefaultGroovyMethods.getText(huntForTheScriptFile(script)));
         } else {
             groovy.parse(script);
         }
