@@ -416,7 +416,7 @@ public final class FindReplaceUtility {
         public FindAction() {
             putValue(Action.NAME, FIND_ACTION_COMMAND);
             putValue(Action.ACTION_COMMAND_KEY, FIND_ACTION_COMMAND);
-            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_F));
+            putValue(Action.MNEMONIC_KEY, Integer.valueOf(KeyEvent.VK_F));
         }
         
         public void actionPerformed(ActionEvent ae) {
@@ -451,7 +451,7 @@ public final class FindReplaceUtility {
         public ReplaceAction() {
             putValue(Action.NAME, REPLACE_ACTION_COMMAND);
             putValue(Action.ACTION_COMMAND_KEY, REPLACE_ACTION_COMMAND);
-            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_R));
+            putValue(Action.MNEMONIC_KEY, Integer.valueOf(KeyEvent.VK_R));
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -499,7 +499,7 @@ public final class FindReplaceUtility {
         public ReplaceAllAction() {
             putValue(Action.NAME, REPLACE_ALL_ACTION_COMMAND);
             putValue(Action.ACTION_COMMAND_KEY, REPLACE_ALL_ACTION_COMMAND);
-            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_A));
+            putValue(Action.MNEMONIC_KEY, Integer.valueOf(KeyEvent.VK_A));
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -548,7 +548,7 @@ public final class FindReplaceUtility {
         public CloseAction() {
             putValue(Action.NAME, CLOSE_ACTION_COMMAND);
             putValue(Action.ACTION_COMMAND_KEY, CLOSE_ACTION_COMMAND);
-            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
+            putValue(Action.MNEMONIC_KEY, Integer.valueOf(KeyEvent.VK_C));
             putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ESCAPE"));
         }
 
