@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -49,18 +49,46 @@ import org.xml.sax.XMLReader;
 import org.xml.sax.helpers.DefaultHandler;
 
 /**
+ * <p>Parse XML into a document tree that may be traversed similar to XPath 
+ * expressions.  For example:</p>
+ * <pre>
+ * def rootNode = new XmlSlurper().parseText( 
+ *    '&lt;root&gt;&lt;one a1="uno!"/&gt;&lt;two&gt;Some text!&lt;/two&gt;&lt;/root&gt;' )
+ *    
+ * assert rootNode.name() == 'root'
+ * assert rootNode.one[0].@a1 == 'uno!'
+ * assert rootNode.two.text() == 'Some text!'
+ * rootNode.children().each { assert it.name() in ['one','two'] }  
+ * </pre>
+ * 
+ * <p>Note that in some cases, a 'selector' expression may not resolve to a 
+ * single node.  For example: </p>
+ * <pre>
+ * def rootNode = new XmlSlurper().parseText( 
+ *    '''&lt;root&gt;
+ *         &lt;a&gt;one!&lt;/a&gt;
+ *         &lt;a&gt;two!&lt;/a&gt;
+ *       &lt;/root&gt;''' )
+ *    
+ * assert rootNode.a.size() == 2
+ * rootNode.a.each { assert it.text() in ['one!','two!'] }  
+ * </pre>
+ * @see GPathResult
  * @author John Wilson
- *
  */
-
 public class XmlSlurper extends DefaultHandler {
   private final XMLReader reader;
   private Node currentNode = null;
   private final Stack stack = new Stack();
   private final StringBuffer charBuffer = new StringBuffer();
-  private final Map namespaceTagHints = new Hashtable();
+  private final Map<String, String> namespaceTagHints = new Hashtable<String, String>();
   private boolean keepWhitespace = false;
 
+  /**
+   * Uses the defaults of not validating and namespace aware.
+   * @throws ParserConfigurationException
+   * @throws SAXException
+   */
   public XmlSlurper() throws ParserConfigurationException, SAXException {
     this(false, true);
   }
@@ -363,7 +391,7 @@ public class XmlSlurper extends DefaultHandler {
   private void addCdata() {
     if (this.charBuffer.length() != 0) {
       //
-      // This element is preceeded by CDATA if keepWhitespace is false (the default setting) and 
+      // This element is preceded by CDATA if keepWhitespace is false (the default setting) and
       // it's not whitespace add it to the body
       // Note that, according to the XML spec, we should preserve the CDATA if it's all whitespace
       // but for the sort of work I'm doing ignoring the whitespace is preferable
