@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,13 +21,16 @@ import org.apache.tools.ant.Task;
 import org.apache.tools.ant.types.DirSet;
 import org.apache.tools.ant.types.Path;
 import org.apache.tools.ant.types.PatternSet;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 import org.codehaus.groovy.tools.groovydoc.ClasspathResourceManager;
 import org.codehaus.groovy.tools.groovydoc.FileOutputTool;
 import org.codehaus.groovy.tools.groovydoc.GroovyDocTool;
 import org.codehaus.groovy.tools.groovydoc.LinkArgument;
+import org.codehaus.groovy.tools.groovydoc.gstringTemplates.GroovyDocTemplateInfo;
 
 import java.io.File;
 import java.io.FilenameFilter;
+import java.io.IOException;
 import java.util.*;
 
 /**
@@ -36,8 +39,6 @@ import java.util.*;
  * @version $Id$
  */
 public class Groovydoc extends Task {
-    private static final String TEMPLATE_BASEDIR = "org/codehaus/groovy/tools/groovydoc/gstringTemplates/";
-    private static final String DOCGEN_BASEDIR = "org/codehaus/groovy/tools/";
     private final LoggingHelper log = new LoggingHelper(this);
 
     private Path sourcePath;
@@ -48,17 +49,20 @@ public class Groovydoc extends Task {
     private String docTitle = "Groovy Documentation";
     private String footer = "Groovy Documentation";
     private String header = "Groovy Documentation";
-    private boolean privateScope;
-    private boolean protectedScope;
-    private boolean packageScope;
-    private boolean publicScope;
+    private Boolean privateScope;
+    private Boolean protectedScope;
+    private Boolean packageScope;
+    private Boolean publicScope;
+    private Boolean author;
     private boolean useDefaultExcludes;
     private boolean includeNoSourcePackages;
-    private boolean author;
     private List<DirSet> packageSets;
     private List<String> sourceFilesToDoc;
     private List<LinkArgument> links = new ArrayList<LinkArgument>();
     private File overviewFile;
+    private File styleSheetFile;
+    // dev note: update javadoc comment for #setExtensions(String) if updating below
+    private String extensions = ".java:.groovy:.gv:.gvy:.gsh";
 
     public Groovydoc() {
         packageNames = new ArrayList<String>();
@@ -97,7 +101,6 @@ public class Groovydoc extends Task {
         // todo: maybe tell groovydoc to use file output
     }
 
-
     /**
      * If set to false, author will not be displayed.
      * Currently not used.
@@ -108,6 +111,16 @@ public class Groovydoc extends Task {
         this.author = author;
     }
 
+    /**
+     * A colon-separated list of filename extensions to look for when searching for files to process in a given directory.
+     * Default value: <code>.java:.groovy:.gv:.gvy:.gsh</code>
+     *
+     * @param extensions new value
+     */
+    public void setExtensions(String extensions) {
+        this.extensions = extensions;
+    }
+
     /**
      * Set the package names to be processed.
      *
@@ -150,10 +163,23 @@ public class Groovydoc extends Task {
      *
      * @param file the overview file
      */
-    public void setOverview(java.io.File file) {
+    public void setOverview(File file) {
         overviewFile = file;
     }
 
+    /**
+     * Indicates the access mode or scope of interest: one of public, protected, package, or private.
+     * Package scoped access is ignored for fields of Groovy classes where they correspond to properties.
+     *
+     * @param access one of public, protected, package, or private
+     */
+    public void setAccess(String access) {
+        if ("public".equals(access)) publicScope = true;
+        else if ("protected".equals(access)) protectedScope = true;
+        else if ("package".equals(access)) packageScope = true;
+        else if ("private".equals(access)) privateScope = true;
+    }
+
     /**
      * Indicate whether all classes and
      * members are to be included in the scope processed.
@@ -166,7 +192,6 @@ public class Groovydoc extends Task {
 
     /**
      * Indicate whether only public classes and members are to be included in the scope processed.
-     * Currently not used.
      *
      * @param b true if scope only includes public level classes and members
      */
@@ -176,7 +201,6 @@ public class Groovydoc extends Task {
 
     /**
      * Indicate whether only protected and public classes and members are to be included in the scope processed.
-     * Currently not used.
      *
      * @param b true if scope includes protected level classes and members
      */
@@ -186,7 +210,7 @@ public class Groovydoc extends Task {
 
     /**
      * Indicate whether only package, protected and public classes and members are to be included in the scope processed.
-     * Currently not used.
+     * Package scoped access is ignored for fields of Groovy classes where they correspond to properties.
      *
      * @param b true if scope includes package level classes and members
      */
@@ -216,6 +240,16 @@ public class Groovydoc extends Task {
         this.header = header;
     }
 
+    /**
+     * Specifies a stylesheet file to use. If not specified,
+     * a default one will be generated for you.
+     *
+     * @param styleSheetFile the css stylesheet file to use
+     */
+    public void setStyleSheetFile(File styleSheetFile) {
+        this.styleSheetFile = styleSheetFile;
+    }
+
     /**
      * Add the directories matched by the nested dirsets to the resulting
      * packages list and the base directories of the dirsets to the Path.
@@ -282,13 +316,14 @@ public class Groovydoc extends Task {
                 File pd = new File(baseDir, dir);
                 String[] files = pd.list(new FilenameFilter() {
                     public boolean accept(File dir1, String name) {
-                        return name.endsWith(".java")
-                                || name.endsWith(".groovy")
-                                || name.endsWith(".gv")
-                                || name.endsWith(".gvy")
-                                || name.endsWith(".gsh")
-                                || (!includeNoSourcePackages
-                                && name.equals("package.html"));
+                        if (!includeNoSourcePackages
+                                && name.equals("package.html")) return true;
+                        final StringTokenizer tokenizer = new StringTokenizer(extensions, ":");
+                        while (tokenizer.hasMoreTokens()) {
+                            String ext = tokenizer.nextToken();
+                            if (name.endsWith(ext)) return true;
+                        }
+                        return false;
                     }
                 });
 
@@ -316,7 +351,7 @@ public class Groovydoc extends Task {
                 // Path.list does it for us.
                 sourcePath.createPathElement().setLocation(baseDir);
             } else {
-                log.verbose(baseDir + " doesn\'t contain any packages, dropping it.");
+                log.verbose(baseDir + " doesn't contain any packages, dropping it.");
             }
         }
     }
@@ -325,44 +360,29 @@ public class Groovydoc extends Task {
         List<String> packagesToDoc = new ArrayList<String>();
         Path sourceDirs = new Path(getProject());
         Properties properties = new Properties();
-        properties.put("windowTitle", windowTitle);
-        properties.put("docTitle", docTitle);
-        properties.put("footer", footer);
-        properties.put("header", header);
-        properties.put("privateScope", privateScope);
-        properties.put("protectedScope", protectedScope);
-        properties.put("publicScope", publicScope);
-        properties.put("packageScope", packageScope);
-        properties.put("author", author);
-        properties.put("overviewFile", overviewFile != null ? overviewFile.getAbsolutePath() : "");
+        properties.setProperty("windowTitle", windowTitle);
+        properties.setProperty("docTitle", docTitle);
+        properties.setProperty("footer", footer);
+        properties.setProperty("header", header);
+        checkScopeProperties(properties);
+        properties.setProperty("publicScope", publicScope.toString());
+        properties.setProperty("protectedScope", protectedScope.toString());
+        properties.setProperty("packageScope", packageScope.toString());
+        properties.setProperty("privateScope", privateScope.toString());
+        properties.setProperty("author", author.toString());
+        properties.setProperty("overviewFile", overviewFile != null ? overviewFile.getAbsolutePath() : "");
 
         if (sourcePath != null) {
             sourceDirs.addExisting(sourcePath);
         }
         parsePackages(packagesToDoc, sourceDirs);
-        
+
         GroovyDocTool htmlTool = new GroovyDocTool(
                 new ClasspathResourceManager(), // we're gonna get the default templates out of the dist jar file
-                sourcePath.list(), // sourcepaths
-                new String[]{ // top level templates
-                        TEMPLATE_BASEDIR + "topLevel/index.html",
-                        TEMPLATE_BASEDIR + "topLevel/overview-frame.html", // needs all package names
-                        TEMPLATE_BASEDIR + "topLevel/allclasses-frame.html", // needs all packages / class names
-                        TEMPLATE_BASEDIR + "topLevel/overview-summary.html", // needs all packages
-                        TEMPLATE_BASEDIR + "topLevel/help-doc.html",
-                        TEMPLATE_BASEDIR + "topLevel/index-all.html",
-                        TEMPLATE_BASEDIR + "topLevel/deprecated-list.html",
-                        TEMPLATE_BASEDIR + "topLevel/stylesheet.css",
-                        TEMPLATE_BASEDIR + "topLevel/inherit.gif",
-                        DOCGEN_BASEDIR + "groovy.ico",
-                },
-                new String[]{ // package level templates
-                        TEMPLATE_BASEDIR + "packageLevel/package-frame.html",
-                        TEMPLATE_BASEDIR + "packageLevel/package-summary.html"
-                },
-                new String[]{ // class level templates
-                        TEMPLATE_BASEDIR + "classLevel/classDocName.html"
-                },
+                sourcePath.list(),
+                GroovyDocTemplateInfo.DEFAULT_DOC_TEMPLATES,
+                GroovyDocTemplateInfo.DEFAULT_PACKAGE_TEMPLATES,
+                GroovyDocTemplateInfo.DEFAULT_CLASS_TEMPLATES,
                 links,
                 properties
         );
@@ -374,6 +394,31 @@ public class Groovydoc extends Task {
         } catch (Exception e) {
             e.printStackTrace();
         }
+        // try to override the default stylesheet with custom specified one if needed
+        if (styleSheetFile != null) {
+            try {
+                String css = DefaultGroovyMethods.getText(styleSheetFile);
+                File outfile = new File(destDir, "stylesheet.css");
+                DefaultGroovyMethods.setText(outfile, css);
+            } catch (IOException e) {
+                System.out.println("Warning: Unable to copy specified stylesheet '" + styleSheetFile.getAbsolutePath() +
+                        "'. Using default stylesheet instead. Due to: " + e.getMessage());
+            }
+        }
+    }
+
+    private void checkScopeProperties(Properties properties) {
+        // make protected the default scope and check for invalid duplication
+        int scopeCount = 0;
+        if (packageScope) scopeCount++;
+        if (privateScope) scopeCount++;
+        if (protectedScope) scopeCount++;
+        if (publicScope) scopeCount++;
+        if (scopeCount == 0) {
+            protectedScope = true;
+        } else if (scopeCount > 1) {
+            throw new BuildException("More than one of public, private, package, or protected scopes specified.");
+        }
     }
 
     /**
