@@ -75,7 +75,7 @@ public class GroovyLexer extends antlr.CharScanner implements GroovyTokenTypes,
     protected int lastSigTokenType = EOF;  // last returned non-whitespace token
 
     public void setTokenObjectClass(String name) {/*ignore*/}
-    
+
     protected Token makeToken(int t) {
         GroovySourceToken tok = new GroovySourceToken(t);
         tok.setColumn(inputState.getTokenStartColumn());
@@ -84,13 +84,13 @@ public class GroovyLexer extends antlr.CharScanner implements GroovyTokenTypes,
         tok.setLineLast(inputState.getLine());
         return tok;
     }
-    
+
     protected void pushParenLevel() {
         parenLevelStack.add(Integer.valueOf(parenLevel*SCS_LIMIT + stringCtorState));
         parenLevel = 0;
         stringCtorState = 0;
     }
-    
+
     protected void popParenLevel() {
         int npl = parenLevelStack.size();
         if (npl == 0)  return;
@@ -482,6 +482,14 @@ tryAgain:
 						mCOMPARE_TO(true);
 						theRetToken=_returnToken;
 					}
+					else if ((LA(1)=='=') && (LA(2)=='=') && (LA(3)=='=')) {
+						mIDENTICAL(true);
+						theRetToken=_returnToken;
+					}
+					else if ((LA(1)=='!') && (LA(2)=='=') && (LA(3)=='=')) {
+						mNOT_IDENTICAL(true);
+						theRetToken=_returnToken;
+					}
 					else if ((LA(1)=='>') && (LA(2)=='>') && (LA(3)=='=')) {
 						mSR_ASSIGN(true);
 						theRetToken=_returnToken;
@@ -514,7 +522,7 @@ tryAgain:
 						mEQUAL(true);
 						theRetToken=_returnToken;
 					}
-					else if ((LA(1)=='!') && (LA(2)=='=')) {
+					else if ((LA(1)=='!') && (LA(2)=='=') && (true)) {
 						mNOT_EQUAL(true);
 						theRetToken=_returnToken;
 					}
@@ -894,6 +902,19 @@ tryAgain:
 		_returnToken = _token;
 	}
 	
+	public final void mIDENTICAL(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
+		int _ttype; Token _token=null; int _begin=text.length();
+		_ttype = IDENTICAL;
+		int _saveIndex;
+		
+		match("===");
+		if ( _createToken && _token==null && _ttype!=Token.SKIP ) {
+			_token = makeToken(_ttype);
+			_token.setText(new String(text.getBuffer(), _begin, text.length()-_begin));
+		}
+		_returnToken = _token;
+	}
+	
 	public final void mLNOT(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
 		int _ttype; Token _token=null; int _begin=text.length();
 		_ttype = LNOT;
@@ -933,6 +954,19 @@ tryAgain:
 		_returnToken = _token;
 	}
 	
+	public final void mNOT_IDENTICAL(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
+		int _ttype; Token _token=null; int _begin=text.length();
+		_ttype = NOT_IDENTICAL;
+		int _saveIndex;
+		
+		match("!==");
+		if ( _createToken && _token==null && _ttype!=Token.SKIP ) {
+			_token = makeToken(_ttype);
+			_token.setText(new String(text.getBuffer(), _begin, text.length()-_begin));
+		}
+		_returnToken = _token;
+	}
+	
 	protected final void mDIV(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
 		int _ttype; Token _token=null; int _begin=text.length();
 		_ttype = DIV;
@@ -1511,8 +1545,8 @@ tryAgain:
 		int _saveIndex;
 		
 		{
-		int _cnt606=0;
-		_loop606:
+		int _cnt608=0;
+		_loop608:
 		do {
 			if ((LA(1)=='\\') && (LA(2)=='\n'||LA(2)=='\r') && (true) && (true)) {
 				match('\\');
@@ -1528,10 +1562,10 @@ tryAgain:
 				match('\f');
 			}
 			else {
-				if ( _cnt606>=1 ) { break _loop606; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+				if ( _cnt608>=1 ) { break _loop608; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 			}
 			
-			_cnt606++;
+			_cnt608++;
 		} while (true);
 		}
 		if ( inputState.guessing==0 ) {
@@ -1594,8 +1628,8 @@ tryAgain:
 		{
 		if (((LA(1)=='\t'||LA(1)=='\n'||LA(1)=='\u000c'||LA(1)=='\r'||LA(1)==' '||LA(1)=='/'||LA(1)=='\\'))&&(!whitespaceIncluded)) {
 			{
-			int _cnt612=0;
-			_loop612:
+			int _cnt614=0;
+			_loop614:
 			do {
 				switch ( LA(1)) {
 				case '\n':  case '\r':
@@ -1616,10 +1650,10 @@ tryAgain:
 						mML_COMMENT(false);
 					}
 				else {
-					if ( _cnt612>=1 ) { break _loop612; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+					if ( _cnt614>=1 ) { break _loop614; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 				}
 				}
-				_cnt612++;
+				_cnt614++;
 			} while (true);
 			}
 		}
@@ -1653,7 +1687,7 @@ tryAgain:
 		
 		match("//");
 		{
-		_loop616:
+		_loop618:
 		do {
 			if ((_tokenSet_1.member(LA(1))) && (true) && (true) && (true)) {
 				{
@@ -1661,7 +1695,7 @@ tryAgain:
 				}
 			}
 			else {
-				break _loop616;
+				break _loop618;
 			}
 			
 		} while (true);
@@ -1683,12 +1717,12 @@ tryAgain:
 		
 		match("/*");
 		{
-		_loop626:
+		_loop628:
 		do {
-			boolean synPredMatched624 = false;
+			boolean synPredMatched626 = false;
 			if (((LA(1)=='*') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0000' && LA(3) <= '\ufffe')) && (true))) {
-				int _m624 = mark();
-				synPredMatched624 = true;
+				int _m626 = mark();
+				synPredMatched626 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1697,12 +1731,12 @@ tryAgain:
 					}
 				}
 				catch (RecognitionException pe) {
-					synPredMatched624 = false;
+					synPredMatched626 = false;
 				}
-				rewind(_m624);
+				rewind(_m626);
 inputState.guessing--;
 			}
-			if ( synPredMatched624 ) {
+			if ( synPredMatched626 ) {
 				match('*');
 			}
 			else if ((LA(1)=='\n'||LA(1)=='\r')) {
@@ -1714,7 +1748,7 @@ inputState.guessing--;
 				}
 			}
 			else {
-				break _loop626;
+				break _loop628;
 			}
 			
 		} while (true);
@@ -1739,7 +1773,7 @@ inputState.guessing--;
 		  throw new SemanticException("getLine() == 1 && getColumn() == 1");
 		match("#!");
 		{
-		_loop620:
+		_loop622:
 		do {
 			if ((_tokenSet_1.member(LA(1)))) {
 				{
@@ -1747,7 +1781,7 @@ inputState.guessing--;
 				}
 			}
 			else {
-				break _loop620;
+				break _loop622;
 			}
 			
 		} while (true);
@@ -1768,10 +1802,10 @@ inputState.guessing--;
 		int _saveIndex;
 		int tt=0;
 		
-		boolean synPredMatched629 = false;
+		boolean synPredMatched631 = false;
 		if (((LA(1)=='\'') && (LA(2)=='\'') && (LA(3)=='\'') && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
-			int _m629 = mark();
-			synPredMatched629 = true;
+			int _m631 = mark();
+			synPredMatched631 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1779,17 +1813,17 @@ inputState.guessing--;
 				}
 			}
 			catch (RecognitionException pe) {
-				synPredMatched629 = false;
+				synPredMatched631 = false;
 			}
-			rewind(_m629);
+			rewind(_m631);
 inputState.guessing--;
 		}
-		if ( synPredMatched629 ) {
+		if ( synPredMatched631 ) {
 			_saveIndex=text.length();
 			match("'''");
 			text.setLength(_saveIndex);
 			{
-			_loop634:
+			_loop636:
 			do {
 				switch ( LA(1)) {
 				case '\\':
@@ -1813,10 +1847,10 @@ inputState.guessing--;
 					break;
 				}
 				default:
-					boolean synPredMatched633 = false;
+					boolean synPredMatched635 = false;
 					if (((LA(1)=='\'') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0000' && LA(3) <= '\ufffe')) && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
-						int _m633 = mark();
-						synPredMatched633 = true;
+						int _m635 = mark();
+						synPredMatched635 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -1837,19 +1871,19 @@ inputState.guessing--;
 							}
 						}
 						catch (RecognitionException pe) {
-							synPredMatched633 = false;
+							synPredMatched635 = false;
 						}
-						rewind(_m633);
+						rewind(_m635);
 inputState.guessing--;
 					}
-					if ( synPredMatched633 ) {
+					if ( synPredMatched635 ) {
 						match('\'');
 					}
 					else if ((_tokenSet_4.member(LA(1)))) {
 						mSTRING_CH(false);
 					}
 				else {
-					break _loop634;
+					break _loop636;
 				}
 				}
 			} while (true);
@@ -1859,10 +1893,10 @@ inputState.guessing--;
 			text.setLength(_saveIndex);
 		}
 		else {
-			boolean synPredMatched638 = false;
+			boolean synPredMatched640 = false;
 			if (((LA(1)=='"') && (LA(2)=='"') && (LA(3)=='"') && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
-				int _m638 = mark();
-				synPredMatched638 = true;
+				int _m640 = mark();
+				synPredMatched640 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1870,12 +1904,12 @@ inputState.guessing--;
 					}
 				}
 				catch (RecognitionException pe) {
-					synPredMatched638 = false;
+					synPredMatched640 = false;
 				}
-				rewind(_m638);
+				rewind(_m640);
 inputState.guessing--;
 			}
-			if ( synPredMatched638 ) {
+			if ( synPredMatched640 ) {
 				_saveIndex=text.length();
 				match("\"\"\"");
 				text.setLength(_saveIndex);
@@ -1892,7 +1926,7 @@ inputState.guessing--;
 					++suppressNewline;
 				}
 				{
-				_loop636:
+				_loop638:
 				do {
 					switch ( LA(1)) {
 					case '\\':
@@ -1915,7 +1949,7 @@ inputState.guessing--;
 							mSTRING_CH(false);
 						}
 					else {
-						break _loop636;
+						break _loop638;
 					}
 					}
 				} while (true);
@@ -2039,17 +2073,17 @@ inputState.guessing--;
 			case 'u':
 			{
 				{
-				int _cnt663=0;
-				_loop663:
+				int _cnt665=0;
+				_loop665:
 				do {
 					if ((LA(1)=='u')) {
 						match('u');
 					}
 					else {
-						if ( _cnt663>=1 ) { break _loop663; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+						if ( _cnt665>=1 ) { break _loop665; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 					}
 					
-					_cnt663++;
+					_cnt665++;
 				} while (true);
 				}
 				if ( inputState.guessing==0 ) {
@@ -2170,7 +2204,7 @@ inputState.guessing--;
 		boolean dollarOK = false;
 		
 		{
-		_loop644:
+		_loop646:
 		do {
 			switch ( LA(1)) {
 			case '\\':
@@ -2189,10 +2223,10 @@ inputState.guessing--;
 				break;
 			}
 			default:
-				boolean synPredMatched643 = false;
+				boolean synPredMatched645 = false;
 				if ((((LA(1)=='"') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true))&&(tripleQuote))) {
-					int _m643 = mark();
-					synPredMatched643 = true;
+					int _m645 = mark();
+					synPredMatched645 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -2213,19 +2247,19 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched643 = false;
+						synPredMatched645 = false;
 					}
-					rewind(_m643);
+					rewind(_m645);
 inputState.guessing--;
 				}
-				if ( synPredMatched643 ) {
+				if ( synPredMatched645 ) {
 					match('"');
 				}
 				else if ((_tokenSet_4.member(LA(1)))) {
 					mSTRING_CH(false);
 				}
 			else {
-				break _loop644;
+				break _loop646;
 			}
 			}
 		} while (true);
@@ -2401,13 +2435,13 @@ inputState.guessing--;
 		
 		}
 		{
-		_loop659:
+		_loop661:
 		do {
 			if ((LA(1)=='*')) {
 				match('*');
 			}
 			else {
-				break _loop659;
+				break _loop661;
 			}
 			
 		} while (true);
@@ -2428,7 +2462,7 @@ inputState.guessing--;
 		int _saveIndex;
 		
 		{
-		_loop653:
+		_loop655:
 		do {
 			if (((LA(1)=='$') && (_tokenSet_2.member(LA(2))))&&(!atValidDollarEscape())) {
 				match('$');
@@ -2437,7 +2471,7 @@ inputState.guessing--;
 				mREGEXP_SYMBOL(false);
 			}
 			else {
-				break _loop653;
+				break _loop655;
 			}
 			
 		} while (true);
@@ -2562,7 +2596,7 @@ inputState.guessing--;
 			
 			}
 			{
-			_loop675:
+			_loop677:
 			do {
 				switch ( LA(1)) {
 				case '0':  case '1':  case '2':  case '3':
@@ -2582,7 +2616,7 @@ inputState.guessing--;
 						mLETTER(false);
 					}
 				else {
-					break _loop675;
+					break _loop677;
 				}
 				}
 			} while (true);
@@ -2591,7 +2625,7 @@ inputState.guessing--;
 		else if ((_tokenSet_9.member(LA(1))) && (true) && (true) && (true)) {
 			mLETTER(false);
 			{
-			_loop677:
+			_loop679:
 			do {
 				if ((_tokenSet_9.member(LA(1)))) {
 					mLETTER(false);
@@ -2600,7 +2634,7 @@ inputState.guessing--;
 					mDIGIT(false);
 				}
 				else {
-					break _loop677;
+					break _loop679;
 				}
 				
 			} while (true);
@@ -2797,40 +2831,40 @@ inputState.guessing--;
 					isDecimal = false;
 				}
 				{
-				int _cnt685=0;
-				_loop685:
+				int _cnt687=0;
+				_loop687:
 				do {
 					if ((_tokenSet_10.member(LA(1))) && (true) && (true) && (true)) {
 						mHEX_DIGIT(false);
 					}
 					else {
-						if ( _cnt685>=1 ) { break _loop685; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+						if ( _cnt687>=1 ) { break _loop687; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 					}
 					
-					_cnt685++;
+					_cnt687++;
 				} while (true);
 				}
 			}
 			else {
-				boolean synPredMatched691 = false;
+				boolean synPredMatched693 = false;
 				if ((((LA(1) >= '0' && LA(1) <= '9')) && (true) && (true) && (true))) {
-					int _m691 = mark();
-					synPredMatched691 = true;
+					int _m693 = mark();
+					synPredMatched693 = true;
 					inputState.guessing++;
 					try {
 						{
 						{
-						int _cnt688=0;
-						_loop688:
+						int _cnt690=0;
+						_loop690:
 						do {
 							if (((LA(1) >= '0' && LA(1) <= '9'))) {
 								matchRange('0','9');
 							}
 							else {
-								if ( _cnt688>=1 ) { break _loop688; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+								if ( _cnt690>=1 ) { break _loop690; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 							}
 							
-							_cnt688++;
+							_cnt690++;
 						} while (true);
 						}
 						{
@@ -2862,34 +2896,18 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched691 = false;
+						synPredMatched693 = false;
 					}
-					rewind(_m691);
+					rewind(_m693);
 inputState.guessing--;
 				}
-				if ( synPredMatched691 ) {
-					{
-					int _cnt693=0;
-					_loop693:
-					do {
-						if (((LA(1) >= '0' && LA(1) <= '9'))) {
-							matchRange('0','9');
-						}
-						else {
-							if ( _cnt693>=1 ) { break _loop693; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
-						}
-						
-						_cnt693++;
-					} while (true);
-					}
-				}
-				else if (((LA(1) >= '0' && LA(1) <= '7')) && (true) && (true) && (true)) {
+				if ( synPredMatched693 ) {
 					{
 					int _cnt695=0;
 					_loop695:
 					do {
-						if (((LA(1) >= '0' && LA(1) <= '7'))) {
-							matchRange('0','7');
+						if (((LA(1) >= '0' && LA(1) <= '9'))) {
+							matchRange('0','9');
 						}
 						else {
 							if ( _cnt695>=1 ) { break _loop695; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
@@ -2898,6 +2916,22 @@ inputState.guessing--;
 						_cnt695++;
 					} while (true);
 					}
+				}
+				else if (((LA(1) >= '0' && LA(1) <= '7')) && (true) && (true) && (true)) {
+					{
+					int _cnt697=0;
+					_loop697:
+					do {
+						if (((LA(1) >= '0' && LA(1) <= '7'))) {
+							matchRange('0','7');
+						}
+						else {
+							if ( _cnt697>=1 ) { break _loop697; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+						}
+						
+						_cnt697++;
+					} while (true);
+					}
 					if ( inputState.guessing==0 ) {
 						isDecimal = false;
 					}
@@ -2916,13 +2950,13 @@ inputState.guessing--;
 				matchRange('1','9');
 				}
 				{
-				_loop698:
+				_loop700:
 				do {
 					if (((LA(1) >= '0' && LA(1) <= '9'))) {
 						matchRange('0','9');
 					}
 					else {
-						break _loop698;
+						break _loop700;
 					}
 					
 				} while (true);
@@ -2999,10 +3033,10 @@ inputState.guessing--;
 				break;
 			}
 			default:
-				boolean synPredMatched704 = false;
+				boolean synPredMatched706 = false;
 				if ((((LA(1)=='.'||LA(1)=='D'||LA(1)=='E'||LA(1)=='F'||LA(1)=='d'||LA(1)=='e'||LA(1)=='f'))&&(isDecimal))) {
-					int _m704 = mark();
-					synPredMatched704 = true;
+					int _m706 = mark();
+					synPredMatched706 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -3022,29 +3056,29 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched704 = false;
+						synPredMatched706 = false;
 					}
-					rewind(_m704);
+					rewind(_m706);
 inputState.guessing--;
 				}
-				if ( synPredMatched704 ) {
+				if ( synPredMatched706 ) {
 					{
 					switch ( LA(1)) {
 					case '.':
 					{
 						match('.');
 						{
-						int _cnt707=0;
-						_loop707:
+						int _cnt709=0;
+						_loop709:
 						do {
 							if (((LA(1) >= '0' && LA(1) <= '9'))) {
 								matchRange('0','9');
 							}
 							else {
-								if ( _cnt707>=1 ) { break _loop707; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+								if ( _cnt709>=1 ) { break _loop709; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 							}
 							
-							_cnt707++;
+							_cnt709++;
 						} while (true);
 						}
 						{
@@ -3199,17 +3233,17 @@ inputState.guessing--;
 		}
 		}
 		{
-		int _cnt716=0;
-		_loop716:
+		int _cnt718=0;
+		_loop718:
 		do {
 			if (((LA(1) >= '0' && LA(1) <= '9'))) {
 				matchRange('0','9');
 			}
 			else {
-				if ( _cnt716>=1 ) { break _loop716; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+				if ( _cnt718>=1 ) { break _loop718; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 			}
 			
-			_cnt716++;
+			_cnt718++;
 		} while (true);
 		}
 		if ( _createToken && _token==null && _ttype!=Token.SKIP ) {
