@@ -24,7 +24,7 @@ import org.objectweb.asm.Opcodes;
  * Represents a field (member variable)
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 11961 $
+ * @version $Revision: 16723 $
  */
 public class FieldNode extends AnnotatedNode implements Opcodes, Variable {
 
@@ -100,6 +100,28 @@ public class FieldNode extends AnnotatedNode implements Opcodes, Variable {
     public boolean isStatic() {
         return (modifiers & ACC_STATIC) != 0;
     }
+
+    /**
+     * @return true if the field is an enum
+     */
+    public boolean isEnum() {
+        return (modifiers & ACC_ENUM) != 0;
+    }
+
+    /**
+     * @return true if the field is final
+     */
+    public boolean isFinal() {
+        return (modifiers & ACC_FINAL) != 0;
+    }
+
+    /**
+     * @return true if the field is public
+     */
+    public boolean isPublic() {
+        return (modifiers & ACC_PUBLIC) != 0;
+    }
+
 	/**
 	 * @param owner The owner to set.
 	 */
@@ -132,4 +154,9 @@ public class FieldNode extends AnnotatedNode implements Opcodes, Variable {
 	public ClassNode getOriginType() {
 		return getType();
 	}
+
+    public void rename(String name) {
+        declaringClass.renameField(this.name,name);
+        this.name = name;
+    }
 }
