@@ -15,15 +15,19 @@
  */
 package org.codehaus.groovy.ast;
 
+import java.util.LinkedList;
+
 /**
  * Represents an inner class declaration
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 13563 $
+ * @version $Revision: 19090 $
  */
 public class InnerClassNode extends ClassNode {
 
     private ClassNode outerClass;
+    private VariableScope scope;
+    private boolean anonymous;
 
     /**
      * @param name is the full name of the class
@@ -42,6 +46,10 @@ public class InnerClassNode extends ClassNode {
     public InnerClassNode(ClassNode outerClass, String name, int modifiers, ClassNode superClass, ClassNode[] interfaces, MixinNode[] mixins) {
         super(name, modifiers, superClass, interfaces, mixins);
         this.outerClass = outerClass;
+
+        if (outerClass.innerClasses == null)
+            outerClass.innerClasses = new LinkedList<InnerClassNode> ();
+        outerClass.innerClasses.add(this);
     }
 
     public ClassNode getOuterClass() {
@@ -54,4 +62,20 @@ public class InnerClassNode extends ClassNode {
     public FieldNode getOuterField(String name) {
         return outerClass.getDeclaredField(name);
     }
+    
+    public VariableScope getVariableScope() {
+        return scope;        
+    }
+    
+    public void setVariableScope(VariableScope scope) {
+        this.scope = scope;
+    }
+
+	public boolean isAnonymous() {
+		return anonymous;
+	}
+
+	public void setAnonymous(boolean anonymous) {
+		this.anonymous = anonymous;
+	}
 }
