@@ -27,7 +27,6 @@ import org.codehaus.groovy.transform.GroovyASTTransformation;
 import org.objectweb.asm.Opcodes;
 
 import java.util.Arrays;
-import java.util.Iterator;
 
 @GroovyASTTransformation(phase = CompilePhase.CANONICALIZATION)
 public class MixinASTTransformation implements ASTTransformation {
@@ -54,8 +53,7 @@ public class MixinASTTransformation implements ASTTransformation {
             useClasses = expr;
         } else if (expr instanceof ListExpression) {
             ListExpression listExpression = (ListExpression) expr;
-            for (Iterator it = listExpression.getExpressions().iterator(); it.hasNext(); ) {
-                Expression ex = (Expression) it.next();
+            for (Expression ex : listExpression.getExpressions()) {
                 if (!(ex instanceof ClassExpression))
                     return;
             }
