@@ -24,8 +24,6 @@ import org.codehaus.groovy.ast.stmt.CatchStatement;
 import org.codehaus.groovy.ast.stmt.ForStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.control.SourceUnit;
-import org.codehaus.groovy.syntax.Types;
-import org.objectweb.asm.Opcodes;
 
 import java.util.LinkedList;
 
@@ -46,7 +44,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     private boolean inPropertyExpression = false;
     private boolean isSpecialConstructorCall = false;
     private boolean inConstructor = false;
-    private boolean inStaticConstructor = false;
 
     private LinkedList stateStack = new LinkedList();
 
@@ -405,6 +402,13 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     // ------------------------------
 
     public void visitClass(ClassNode node) {
+    	// AIC are already done, doing them here again will lead
+    	// to wrong scopes
+    	if (node instanceof InnerClassNode) {
+    		InnerClassNode in = (InnerClassNode) node;
+    		if (in.isAnonymous()) return;
+    	}
+    	
         pushState();
 
         currentClass = node;
@@ -417,7 +421,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     protected void visitConstructorOrMethod(MethodNode node, boolean isConstructor) {
         pushState(node.isStatic());
         inConstructor = isConstructor;
-        inStaticConstructor = node.isStaticConstructor();
         node.setVariableScope(currentScope);
         
         // GROOVY-2156
@@ -490,55 +493,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         popState();
     }
     
-    public void visitBinaryExpression(BinaryExpression expression) {
-        super.visitBinaryExpression(expression);
-        switch (expression.getOperation().getType()){
-            case Types.EQUAL: // = assignment
-            case Types.BITWISE_AND_EQUAL:
-            case Types.BITWISE_OR_EQUAL:
-            case Types.BITWISE_XOR_EQUAL:
-            case Types.PLUS_EQUAL:
-            case Types.MINUS_EQUAL:
-            case Types.MULTIPLY_EQUAL:
-            case Types.DIVIDE_EQUAL:
-            case Types.INTDIV_EQUAL:
-            case Types.MOD_EQUAL:
-            case Types.POWER_EQUAL:
-            case Types.LEFT_SHIFT_EQUAL:
-            case Types.RIGHT_SHIFT_EQUAL:
-            case Types.RIGHT_SHIFT_UNSIGNED_EQUAL:
-                checkFinalFieldAccess(expression.getLeftExpression());
-                break;
-            default: break;
-        }
-        
-    }
-
-    private void checkFinalFieldAccess(Expression expression) {
-        if (!(expression instanceof VariableExpression)) return;
-        boolean isFinal, isStatic, error;
-        int modifiers = 0;
-        VariableExpression ve = (VariableExpression) expression;
-        Variable v = ve.getAccessedVariable();
-        if (v instanceof PropertyNode) {
-            PropertyNode pn = (PropertyNode) v;
-            modifiers = pn.getModifiers();
-        } else if (v instanceof FieldNode) {
-            FieldNode fn = (FieldNode) v;
-            modifiers = fn.getModifiers();
-        }
-        /*
-         *  if it is static final but not accessed inside a static constructor, or,
-         *  if it is an instance final but not accessed inside a instance constructor, it is an error
-         */
-        isFinal = (modifiers & Opcodes.ACC_FINAL) != 0;
-        isStatic = (modifiers & Opcodes.ACC_STATIC) != 0;
-        error = isFinal && ((isStatic && !inStaticConstructor) || (!isStatic && !inConstructor));
-
-        if (error) addError("cannnot access final field or property " +
-                            "outside of constructor.", expression);
-    }
-
     public void visitProperty(PropertyNode node) {
         pushState(node.isStatic());
         super.visitProperty(node);
