@@ -1,27 +1,18 @@
 /*
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.  
- * Use is subject to license terms.
+ * Copyright 2003-2010 the original author or authors.
  *
- * Redistribution and use in source and binary forms, with or without modification, are 
- * permitted provided that the following conditions are met: Redistributions of source code 
- * must retain the above copyright notice, this list of conditions and the following disclaimer.
- * Redistributions in binary form must reproduce the above copyright notice, this list of 
- * conditions and the following disclaimer in the documentation and/or other materials 
- * provided with the distribution. Neither the name of the Sun Microsystems nor the names of 
- * is contributors may be used to endorse or promote products derived from this software 
- * without specific prior written permission. 
-
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
- * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-
 package org.codehaus.groovy.jsr223;
 
 import groovy.lang.GroovySystem;
@@ -33,9 +24,13 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 
-/*
- * @author Mike Grogan
- * @author A. Sundararajan
+/**
+ * A factory class conforming to JSR-223 which is used to instantiate
+ * Groovy <code>ScriptEngines</code> and also exposes metadata describing
+ * Groovy's engine class.
+ *
+ * @author Adapted from original by Mike Grogan
+ * @author Adapted from original by A. Sundararajan
  * @author Jim White
  * @author Guillaume Laforge
  */
@@ -51,21 +46,20 @@ public class GroovyScriptEngineFactory implements ScriptEngineFactory {
         return "Groovy Scripting Engine";
     }
 
-/**
-   Note that the scripting.dev.java.net engine had this backwards.
-   The engine version refers to this engine implementation.
-   Whereas language version refers to the groovy implementation
-   (which is obtained from the runtime).
-*/
-
+    /**
+     * Note that the scripting.dev.java.net engine had this backwards.
+     * The engine version refers to this engine implementation.
+     * Whereas language version refers to the groovy implementation
+     * (which is obtained from the runtime).
+     */
     public String getEngineVersion() {
         return VERSION;
     }
 
-/**
-    This is also different than scripting.dev.java.net which used an
-    initial lowercase.  But these are proper names and should be capitalized.
-*/
+    /**
+     *   This is also different than scripting.dev.java.net which used an
+     *   initial lowercase.  But these are proper names and should be capitalized.
+     */
     public String getLanguageName() {
         return LANGUAGE_NAME;
     }
@@ -106,12 +100,11 @@ public class GroovyScriptEngineFactory implements ScriptEngineFactory {
     
     }
     
-   
     public ScriptEngine getScriptEngine() {
         return new GroovyScriptEngineImpl();
     }
     
-     public String getMethodCallSyntax(String obj, String method, 
+    public String getMethodCallSyntax(String obj, String method,
                                         String... args) {
         
         String ret = obj + "." + method + "(";
