@@ -15,13 +15,15 @@
  */
 package org.codehaus.groovy.reflection.stdclasses;
 
+import org.codehaus.groovy.reflection.ClassInfo;
+
 /**
  * @author Alex.Tkachman
  */
 public class ByteCachedClass extends NumberCachedClass {
 	private boolean allowNull;
-    public ByteCachedClass(Class klazz, boolean allowNull) {
-        super(klazz);
+    public ByteCachedClass(Class klazz, ClassInfo classInfo, boolean allowNull) {
+        super(klazz, classInfo);
         this.allowNull = allowNull;
     }
 
@@ -30,7 +32,10 @@ public class ByteCachedClass extends NumberCachedClass {
             return argument;
         }
 
-        return new Byte(((Number) argument).byteValue());
+        if (argument instanceof Number) {
+            return new Byte(((Number) argument).byteValue());
+        }
+        return argument;
     }
 
     public boolean isDirectlyAssignable(Object argument) {
