@@ -35,7 +35,7 @@ public class IntegerCachedClass extends NumberCachedClass {  // int, Integer
         }
 
         if (argument instanceof Number) {
-            return Integer.valueOf(((Number) argument).intValue());
+            return new Integer(((Number) argument).intValue());
         }
         return argument;
     }
