@@ -21,6 +21,8 @@ import org.codehaus.groovy.runtime.metaclass.MetaClassRegistryImpl;
 import org.codehaus.groovy.runtime.metaclass.MissingMethodExecutionFailed;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
 import org.codehaus.groovy.runtime.wrappers.PojoWrapper;
+import org.codehaus.groovy.transform.powerassert.PowerAssertionError;
+import org.codehaus.groovy.util.ReleaseInfo;
 import org.w3c.dom.Element;
 
 import java.beans.Introspector;
@@ -35,7 +37,7 @@ import java.util.regex.Pattern;
  * A static helper class to make bytecode generation easier and act as a facade over the Invoker
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 16878 $
+ * @version $Revision: 18400 $
  */
 public class InvokerHelper {
     private   static final Object[] EMPTY_MAIN_ARGS = new Object[]{new String[0]};
@@ -314,6 +316,7 @@ public class InvokerHelper {
      * @param right regular expression to compare the string to
      */
     public static boolean matchRegex(Object left, Object right) {
+    	if(left == null || right == null) return false;
         Pattern pattern;
         if (right instanceof Pattern) {
             pattern = (Pattern) right;
@@ -372,7 +375,7 @@ public class InvokerHelper {
 
     public static void assertFailed(Object expression, Object message) {
         if (message == null || "".equals(message)) {
-            throw new AssertionError("Expression: " + expression);
+            throw new PowerAssertionError(expression.toString());
         }
         throw new AssertionError(String.valueOf(message) + ". Expression: " + expression);
     }
@@ -404,11 +407,11 @@ public class InvokerHelper {
                     // though the bindings will be ignored
                     script = new Script() {
                         public Object run() {
-                        	Object args = getBinding().getVariables().get("args");
-                        	Object argsToPass = EMPTY_MAIN_ARGS;
-                        	if(args != null && args instanceof String[]) {
-                        		argsToPass = args;
-                        	}
+                            Object args = getBinding().getVariables().get("args");
+                            Object argsToPass = EMPTY_MAIN_ARGS;
+                            if(args != null && args instanceof String[]) {
+                                argsToPass = args;
+                            }
                             object.invokeMethod("main", argsToPass);
                             return null;
                         }
@@ -443,18 +446,13 @@ public class InvokerHelper {
         }
     }
 
+    /**
+     * @deprecated Use GroovySystem version instead. 
+     */
     public static String getVersion() {
-        String version = null;
-        Package p = Package.getPackage("groovy.lang");
-        if (p != null) {
-            version = p.getImplementationVersion();
-        }
-        if (version == null) {
-            version = "";
-        }
-        return version;
+        return ReleaseInfo.getVersion();
     }
-
+    
     /**
      * Writes the given object to the given stream
      */
