@@ -32,7 +32,7 @@ import java.util.regex.Pattern;
  * A static helper class to interface bytecode and runtime
  *
  * @author Jochen Theodorou
- * @version $Revision: 17783 $
+ * @version $Revision: 18724 $
  */
 public class ScriptBytecodeAdapter {
     public static final Object[] EMPTY_ARGS = {};
@@ -676,6 +676,10 @@ public class ScriptBytecodeAdapter {
         return left == right;
     }
 
+    public static boolean compareNotIdentical(Object left, Object right) {
+        return left != right;
+    }
+
     public static boolean compareEqual(Object left, Object right) {
         return DefaultTypeTransformation.compareEqual(left, right);
     }
