@@ -17,14 +17,14 @@ package org.codehaus.groovy.runtime.metaclass;
 
 import groovy.lang.MetaMethod;
 import org.codehaus.groovy.reflection.CachedClass;
-import org.codehaus.groovy.reflection.FastArray;
-import org.codehaus.groovy.reflection.SingleKeyHashMap;
+import org.codehaus.groovy.util.FastArray;
+import org.codehaus.groovy.reflection.GeneratedMetaMethod;
+import org.codehaus.groovy.util.SingleKeyHashMap;
 
 import java.util.NoSuchElementException;
 
 public class MetaMethodIndex {
     public SingleKeyHashMap methodHeaders = new SingleKeyHashMap();
-    private CachedClass theCachedClass;
 
     public static class Header {
         public Entry head;
@@ -66,7 +66,6 @@ public class MetaMethodIndex {
     }
 
     public MetaMethodIndex(CachedClass theCachedClass) {
-        this.theCachedClass = theCachedClass;
         init(DEFAULT_CAPACITY);
 
         CachedClass last = null;
@@ -435,7 +434,9 @@ public class MetaMethodIndex {
         return method instanceof NewInstanceMetaMethod ||
                 method instanceof NewStaticMetaMethod ||
                 method instanceof ClosureMetaMethod ||
-                method instanceof ClosureStaticMetaMethod;
+                method instanceof GeneratedMetaMethod ||
+                method instanceof ClosureStaticMetaMethod ||
+                method instanceof MixinInstanceMetaMethod;
     }
 
     private boolean isMatchingMethod(MetaMethod aMethod, MetaMethod method) {
