@@ -41,6 +41,8 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 /*
  *  todo: order methods alphabetically (implement compareTo enough?)
@@ -95,7 +97,7 @@ public class GroovyRootDocBuilder {
         java2groovyTraverser.process(ast);
 
         // now mutate (groovify) the ast into groovy
-        Visitor groovifier = new Groovifier(tokenNames);
+        Visitor groovifier = new Groovifier(tokenNames, false);
         AntlrASTProcessor groovifierTraverser = new PreOrderTraversal(groovifier);
         groovifierTraverser.process(ast);
 
@@ -149,15 +151,22 @@ public class GroovyRootDocBuilder {
         setOverview();
 
         List<File> sourcepathFiles = new ArrayList<File>();
-        for (String sourcepath : sourcepaths) {
-            sourcepathFiles.add(new File(sourcepath).getAbsoluteFile());
+        if (sourcepaths != null) {
+            for (String sourcepath : sourcepaths) {
+                sourcepathFiles.add(new File(sourcepath).getAbsoluteFile());
+            }
         }
 
         for (String filename : filenames) {
+            File srcFile = new File(filename);
+            if (srcFile.exists()) {
+                processFile(filename, srcFile, true);
+                continue;
+            }
             for (File spath : sourcepathFiles) {
-                File srcFile = new File(spath, filename);
+                srcFile = new File(spath, filename);
                 if (srcFile.exists()) {
-                    processFile(filename, srcFile);
+                    processFile(filename, srcFile, false);
                     break;
                 }
             }
@@ -176,15 +185,19 @@ public class GroovyRootDocBuilder {
         }
     }
 
-    private void processFile(String filename, File srcFile) throws IOException {
+    private void processFile(String filename, File srcFile, boolean isAbsolute) throws IOException {
         String src = DefaultGroovyMethods.getText(srcFile);
-        String packagePath = tool.getPath(filename).replace('\\', FS);
+        String packagePath = isAbsolute ? "DefaultPackage" : tool.getPath(filename).replace('\\', FS);
         String file = tool.getFile(filename);
-        SimpleGroovyPackageDoc packageDoc = (SimpleGroovyPackageDoc) rootDoc.packageNamed(packagePath);
-        if (packageDoc == null) {
-            packageDoc = new SimpleGroovyPackageDoc(packagePath);
+        SimpleGroovyPackageDoc packageDoc = null;
+        if (!isAbsolute) {
+            packageDoc = (SimpleGroovyPackageDoc) rootDoc.packageNamed(packagePath);
         }
+        // todo: this might not work correctly for absolute paths
         if (filename.endsWith("package.html") || filename.endsWith("package-info.java") || filename.endsWith("package-info.groovy")) {
+            if (packageDoc == null) {
+                packageDoc = new SimpleGroovyPackageDoc(packagePath);
+            }
             processPackageInfo(src, filename, packageDoc);
             rootDoc.put(packagePath, packageDoc);
             return;
@@ -192,6 +205,16 @@ public class GroovyRootDocBuilder {
         try {
             Map<String, GroovyClassDoc> classDocs = getClassDocsFromSingleSource(packagePath, file, src);
             rootDoc.putAllClasses(classDocs);
+            if (isAbsolute) {
+                final Map.Entry<String, GroovyClassDoc> docEntry = classDocs.entrySet().iterator().next();
+                String fullPath = docEntry.getValue().getFullPathName();
+                int slash = fullPath.lastIndexOf(FS);
+                if (slash > 0) packagePath = fullPath.substring(0, slash);
+                packageDoc = (SimpleGroovyPackageDoc) rootDoc.packageNamed(packagePath);
+            }
+            if (packageDoc == null) {
+                packageDoc = new SimpleGroovyPackageDoc(packagePath);
+            }
             packageDoc.putAll(classDocs);
             rootDoc.put(packagePath, packageDoc);
         } catch (RecognitionException e) {
@@ -213,10 +236,52 @@ public class GroovyRootDocBuilder {
         } else {
             description = trimPackageAndComments(src);
         }
+        description = replaceTags(description, packageDoc);
         packageDoc.setDescription(description);
         return description;
     }
 
+    // TODO remove dup with SimpleGroovyClassDoc
+    private String replaceTags(String orig, SimpleGroovyPackageDoc packageDoc) {
+        String result = orig.replaceAll("(?m)^\\s*\\*", ""); // todo precompile regex
+
+        // {@link processing hack}
+        result = replaceAllTags(result, "", "", SimpleGroovyClassDoc.LINK_REGEX, packageDoc);
+
+        // {@code processing hack}
+        result = replaceAllTags(result, "<TT>", "</TT>", SimpleGroovyClassDoc.CODE_REGEX, packageDoc);
+
+        // hack to reformat other groovydoc block tags (@see, @return, @param, @throws, @author, @since) into html
+        result = replaceAllTags(result + "@endMarker", "<DL><DT><B>$1:</B></DT><DD>", "</DD></DL>", SimpleGroovyClassDoc.TAG_REGEX, packageDoc);
+        // remove @endMarker
+        result = result.substring(0, result.length() - 10);
+
+        return SimpleGroovyClassDoc.decodeSpecialSymbols(result);
+    }
+
+    // TODO remove dup with SimpleGroovyClassDoc
+    private String replaceAllTags(String self, String s1, String s2, Pattern regex, SimpleGroovyPackageDoc packageDoc) {
+        Matcher matcher = regex.matcher(self);
+        if (matcher.find()) {
+            matcher.reset();
+            StringBuffer sb = new StringBuffer();
+            while (matcher.find()) {
+                String tagname = matcher.group(1);
+                if (tagname.equals("see") || tagname.equals("link")) {
+                    matcher.appendReplacement(sb, s1 + SimpleGroovyClassDoc.getDocUrl(
+                            SimpleGroovyClassDoc.encodeSpecialSymbols(matcher.group(2)), false, links, packageDoc.getRelativeRootPath(), rootDoc, null) + s2);
+                } else if (!tagname.equals("interface")) {
+                    matcher.appendReplacement(sb, s1 + SimpleGroovyClassDoc.encodeSpecialSymbols(matcher.group(2)) + s2);
+                }
+            }
+            matcher.appendTail(sb);
+            return sb.toString();
+        } else {
+            return self;
+        }
+    }
+
+
     private void calcThenSetSummary(String src, SimpleGroovyPackageDoc packageDoc) {
         packageDoc.setSummary(SimpleGroovyDoc.calculateFirstSentence(src));
     }
@@ -228,7 +293,7 @@ public class GroovyRootDocBuilder {
 
     private String trimPackageAndComments(String src) {
         return src.replaceFirst("(?sm)^package.*", "")
-                .replaceFirst("(?sm)/\\*\\*(.*)\\*/", "$1")
+                .replaceFirst("(?sm)/.*\\*\\*(.*)\\*/", "$1")
                 .replaceAll("(?m)^\\s*\\*", "");
     }
 
