@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,11 +22,14 @@ import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implements GroovyClassDoc {
-    private static final Pattern TAG_REGEX = Pattern.compile("(?m)@([a-z]+)\\s+(.*$[^@]*)");
-    private static final Pattern LINK_REGEX = Pattern.compile("(?m)[{]@(link)\\s+([^}]*)}");
-    private static final Pattern CODE_REGEX = Pattern.compile("(?m)[{]@(code)\\s+([^}]*)}");
-
+public class SimpleGroovyClassDoc extends SimpleGroovyAbstractableElementDoc implements GroovyClassDoc {
+    public static final Pattern TAG_REGEX = Pattern.compile("(?sm)\\s*@([a-zA-Z.]+)\\s+(.*?)(?=\\s+@)");
+    public static final Pattern LINK_REGEX = Pattern.compile("(?m)[{]@(link)\\s+([^}]*)}");
+    public static final Pattern CODE_REGEX = Pattern.compile("(?m)[{]@(code)\\s+([^}]*)}");
+    public static final Pattern REF_LABEL_REGEX = Pattern.compile("([\\w.#]*(\\(.*\\))?)(\\s(.*))?");
+    public static final Pattern NAME_ARGS_REGEX = Pattern.compile("([^(]+)\\(([^)]*)\\)");
+    public static final Pattern SPLIT_ARGS_REGEX = Pattern.compile(",\\s*");
+    private static final List<String> PRIMITIVES = Arrays.asList("void", "boolean", "byte", "short", "char", "int", "long", "float", "double");
     private final List<GroovyConstructorDoc> constructors;
     private final List<GroovyFieldDoc> fields;
     private final List<GroovyFieldDoc> properties;
@@ -41,6 +44,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
     private GroovyClassDoc outer;
     private String superClassName;
     private String fullPathName;
+    private GroovyRootDoc savedRootDoc = null;
 
     public SimpleGroovyClassDoc(List<String> importedClassesAndPackages, String name, List<LinkArgument> links) {
         super(name);
@@ -250,6 +254,7 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
     }
 
     void resolve(GroovyRootDoc rootDoc) {
+        this.savedRootDoc = rootDoc;
         Map visibleClasses = rootDoc.getVisibleClasses(importedClassesAndPackages);
 
         // resolve constructor parameter types
@@ -261,6 +266,9 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
                 String paramTypeName = param.typeName();
                 if (visibleClasses.containsKey(paramTypeName)) {
                     param.setType((GroovyType) visibleClasses.get(paramTypeName));
+                } else {
+                    GroovyClassDoc doc = resolveClass(rootDoc, paramTypeName);
+                    if (doc != null) param.setType(doc);
                 }
             }
         }
@@ -271,6 +279,9 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
             String typeName = fieldType.typeName();
             if (visibleClasses.containsKey(typeName)) {
                 mutableField.setType((GroovyType) visibleClasses.get(typeName));
+            } else {
+                GroovyClassDoc doc = resolveClass(rootDoc, typeName);
+                if (doc != null) mutableField.setType(doc);
             }
         }
 
@@ -282,6 +293,9 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
             String typeName = returnType.typeName();
             if (visibleClasses.containsKey(typeName)) {
                 method.setReturnType((GroovyType) visibleClasses.get(typeName));
+            } else {
+                GroovyClassDoc doc = resolveClass(rootDoc, typeName);
+                if (doc != null) method.setReturnType(doc);
             }
 
             // parameters
@@ -290,6 +304,9 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
                 String paramTypeName = param.typeName();
                 if (visibleClasses.containsKey(paramTypeName)) {
                     param.setType((GroovyType) visibleClasses.get(paramTypeName));
+                } else {
+                    GroovyClassDoc doc = resolveClass(rootDoc, paramTypeName);
+                    if (doc != null) param.setType(doc);
                 }
             }
         }
@@ -313,18 +330,62 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
     }
 
     public String getDocUrl(String type, boolean full) {
+        return getDocUrl(type, full, links, getRelativeRootPath(), savedRootDoc, this);
+    }
+
+    private static String resolveMethodArgs(GroovyRootDoc rootDoc, SimpleGroovyClassDoc classDoc, String type) {
+        if (type.indexOf("(") < 0) return type;
+        Matcher m = NAME_ARGS_REGEX.matcher(type);
+        if (m.matches()) {
+            String name = m.group(1);
+            String args = m.group(2);
+            StringBuilder sb = new StringBuilder();
+            sb.append(name);
+            sb.append("(");
+            String[] argParts = SPLIT_ARGS_REGEX.split(args);
+            boolean first = true;
+            for (String argPart : argParts) {
+                if (first) first = false;
+                else sb.append(", ");
+                GroovyClassDoc doc = classDoc.resolveClass(rootDoc, argPart);
+                sb.append(doc == null ? argPart : doc.qualifiedTypeName());
+            }
+            sb.append(")");
+            return sb.toString();
+        }
+        return type;
+    }
+
+    public static String getDocUrl(String type, boolean full, List<LinkArgument> links, String relativePath, GroovyRootDoc rootDoc, SimpleGroovyClassDoc classDoc) {
         if (type == null)
             return type;
         type = type.trim();
-        if (type.startsWith("#"))
-            return "<a href='" + type + "'>" + type + "</a>";
-        if (type.endsWith("[]")) {
-            return getDocUrl(type.substring(0, type.length() - 2), full) + "[]";
+        if (isPrimitiveType(type)) return type;
+        if (type.equals("def")) type = "java.lang.Object def";
+
+        String label = null;
+        Matcher matcher = REF_LABEL_REGEX.matcher(type);
+        if (matcher.find()) {
+            type = matcher.group(1);
+            label = matcher.group(4);
         }
-        // TODO move next 4 lines to resolve?
-        if (type.indexOf('.') == -1) {
-            Class c = resolveExternalClassFromImport(type);
+
+        if (type.startsWith("#"))
+            return "<a href='" + resolveMethodArgs(rootDoc, classDoc, type) + "'>" + (label == null ? type.substring(1) : label) + "</a>";
+
+        if (type.endsWith("[]")) {
+            if (label != null)
+                return getDocUrl(type.substring(0, type.length() - 2) + " " + label, full, links, relativePath, rootDoc, classDoc);
+            return getDocUrl(type.substring(0, type.length() - 2), full, links, relativePath, rootDoc, classDoc) + "[]";
+        }
+
+        if (type.indexOf('.') == -1 && classDoc != null) {
+            String[] pieces = type.split("#");
+            String candidate = pieces[0];
+            Class c = classDoc.resolveExternalClassFromImport(candidate);
             if (c != null) type = c.getName();
+            if (pieces.length > 1) type += "#" + pieces[1];
+            type = resolveMethodArgs(rootDoc, classDoc, type);
         }
         if (type.indexOf('.') == -1)
             return type;
@@ -332,20 +393,30 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         final String[] target = type.split("#");
         String shortClassName = target[0].replaceAll(".*\\.", "");
         shortClassName += (target.length > 1 ? "#" + target[1].split("\\(")[0] : "");
-        String name = full ? target[0] : shortClassName;
+        String name = (full ? target[0] : shortClassName).replaceAll("#", ".");
+
+        // last chance lookup for classes within the current codebase
+        if (rootDoc != null) {
+            String slashedName = target[0].replaceAll("\\.", "/");
+            GroovyClassDoc doc = rootDoc.classNamed(slashedName);
+            if (doc != null) {
+                return buildUrl(relativePath, target, label == null ? name : label);
+            }
+        }
+
         for (LinkArgument link : links) {
             final StringTokenizer tokenizer = new StringTokenizer(link.getPackages(), ", ");
             while (tokenizer.hasMoreTokens()) {
                 final String token = tokenizer.nextToken();
                 if (type.startsWith(token)) {
-                    return buildUrl(link.getHref(), target, name);
+                    return buildUrl(link.getHref(), target, label == null ? name : label);
                 }
             }
         }
         return type;
     }
 
-    private String buildUrl(String relativeRoot, String[] target, String shortClassName) {
+    private static String buildUrl(String relativeRoot, String[] target, String shortClassName) {
         if (!relativeRoot.endsWith("/")) {
             relativeRoot += "/";
         }
@@ -354,16 +425,27 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
     }
 
     private GroovyClassDoc resolveClass(GroovyRootDoc rootDoc, String name) {
-        GroovyClassDoc doc = rootDoc.classNamed(name);
+        if (isPrimitiveType(name)) return null;
+        GroovyClassDoc doc = ((SimpleGroovyRootDoc)rootDoc).classNamedExact(name);
         if (doc != null) return doc;
+        int slashIndex = name.lastIndexOf("/");
+        if (slashIndex < 1) {
+            doc = resolveInternalClassDocFromImport(rootDoc, name);
+            if (doc != null) return doc;
+            for (GroovyClassDoc nestedDoc : nested) {
+                if (nestedDoc.name().endsWith("." + name))
+                    return nestedDoc;
+            }
+            doc = rootDoc.classNamed(name);
+            if (doc != null) return doc;
+        }
 
         // The class is not in the tree being documented
         String shortname = name;
-        int slashIndex = name.lastIndexOf("/");
         Class c = null;
         if (slashIndex > 0) {
             shortname = name.substring(slashIndex + 1);
-            c = resolveExternalClass(name);
+            c = resolveExternalFullyQualifiedClass(name);
         } else {
             c = resolveExternalClassFromImport(name);
         }
@@ -377,13 +459,41 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         return placeholder;
     }
 
-    private Class resolveExternalClassFromImport(String name) {
+    private static boolean isPrimitiveType(String name) {
+        String type = name;
+        if (name.endsWith("[]")) type = name.substring(0, name.length() - 2);
+        return PRIMITIVES.contains(type);
+    }
+
+    private GroovyClassDoc resolveInternalClassDocFromImport(GroovyRootDoc rootDoc, String baseName) {
+        if (isPrimitiveType(baseName)) return null;
         for (String importName : importedClassesAndPackages) {
-            if (importName.endsWith("/*")) {
-                String candidate = importName.substring(0, importName.length() - 2).replace('/', '.') + "." + name;
+            if (importName.endsWith("/" + baseName)) {
+                GroovyClassDoc doc = ((SimpleGroovyRootDoc)rootDoc).classNamedExact(importName);
+                if (doc != null) return doc;
+            } else if (importName.endsWith("/*")) {
+                GroovyClassDoc doc = ((SimpleGroovyRootDoc)rootDoc).classNamedExact(importName.substring(0, importName.length() - 2) + baseName);
+                if (doc != null) return doc;
+            }
+        }
+        return null;
+    }
+
+    private Class resolveExternalClassFromImport(String name) {
+        if (isPrimitiveType(name)) return null;
+        for (String importName : importedClassesAndPackages) {
+            String candidate = null;
+            if (importName.endsWith("/" + name)) {
+                candidate = importName.replaceAll("/", ".");
+            } else if (importName.endsWith("/*")) {
+                candidate = importName.substring(0, importName.length() - 2).replace('/', '.') + "." + name;
+            }
+            if (candidate != null) {
                 try {
                     // TODO cache these??
                     return Class.forName(candidate);
+                } catch (NoClassDefFoundError e) {
+                    // ignore
                 } catch (ClassNotFoundException e) {
                     // ignore
                 }
@@ -392,11 +502,13 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         return null;
     }
 
-    private Class resolveExternalClass(String name) {
+    private Class resolveExternalFullyQualifiedClass(String name) {
         String candidate = name.replace('/', '.');
         try {
             // TODO cache these??
             return Class.forName(candidate);
+        } catch (NoClassDefFoundError e) {
+            // ignore
         } catch (ClassNotFoundException e) {
             // ignore
         }
@@ -442,10 +554,6 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         return null;
     }
 
-    public boolean isAbstract() {/*todo*/
-        return false;
-    }
-
     public boolean isExternalizable() {/*todo*/
         return false;
     }
@@ -490,7 +598,8 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
     }
 
     public String qualifiedTypeName() {
-        return fullPathName == null ? "null" : fullPathName.replace('/', '.');
+        String qtnWithSlashes = fullPathName.startsWith("DefaultPackage/") ? fullPathName.substring("DefaultPackage/".length()) : fullPathName;
+        return qtnWithSlashes.replace('/', '.');
     }
 
     // TODO remove dupe with SimpleGroovyType
@@ -509,10 +618,17 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         interfaceNames.add(className);
     }
 
-    public void setRawCommentText(String rawCommentText) {
-        super.setRawCommentText(rawCommentText);
-        setCommentText(replaceTags(rawCommentText));
-     }
+    public String firstSentenceCommentText() {
+        if (super.firstSentenceCommentText() == null)
+            setFirstSentenceCommentText(replaceTags(calculateFirstSentence(getRawCommentText())));
+        return super.firstSentenceCommentText();
+    }
+
+    public String commentText() {
+        if (super.commentText() == null)
+            setCommentText(replaceTags(getRawCommentText()));
+        return super.commentText();
+    }
 
     public String replaceTags(String comment) {
         String result = comment.replaceAll("(?m)^\\s*\\*", ""); // todo precompile regex
@@ -523,9 +639,10 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         // {@code processing hack}
         result = replaceAllTags(result, "<TT>", "</TT>", CODE_REGEX);
 
-        // hack to reformat other groovydoc tags (@see, @return, @link, @param, @throws, @author, @since) into html
-        // todo: replace with proper tag support
-        result = replaceAllTags(result, "<DL><DT><B>$1:</B></DT><DD>", "</DD></DL>", TAG_REGEX);
+        // hack to reformat other groovydoc block tags (@see, @return, @param, @throws, @author, @since) into html
+        result = replaceAllTags(result + "@endMarker", "<DL><DT><B>$1:</B></DT><DD>", "</DD></DL>", TAG_REGEX);
+        // remove @endMarker
+        result = result.substring(0, result.length() - 10);
 
         return decodeSpecialSymbols(result);
     }
@@ -551,11 +668,11 @@ public class SimpleGroovyClassDoc extends SimpleGroovyProgramElementDoc implemen
         }
     }
 
-    private String encodeSpecialSymbols(String text) {
+    public static String encodeSpecialSymbols(String text) {
         return Matcher.quoteReplacement(text.replaceAll("@", "&at;"));
     }
 
-    private String decodeSpecialSymbols(String text) {
+    public static String decodeSpecialSymbols(String text) {
         return text.replaceAll("&at;", "@");
     }
 
