@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,6 +45,8 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
     private boolean insideEnum;
     private Map<String, SimpleGroovyClassDoc> foundClasses;
     private boolean isGroovy;
+    private boolean deferSetup;
+    private String className;
 
     public SimpleGroovyClassDocAssembler(String packagePath, String file, SourceBuffer sourceBuffer, List<LinkArgument> links, Properties properties, boolean isGroovy) {
         this.sourceBuffer = sourceBuffer;
@@ -55,14 +57,20 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
 
         stack = new Stack<GroovySourceAST>();
         classDocs = new HashMap<String, GroovyClassDoc>();
-        String className = file;
+        className = file;
         if (file != null) {
             // todo: replace this simple idea of default class name
             int idx = file.lastIndexOf(".");
             className = file.substring(0, idx);
         }
 
+        deferSetup = packagePath.equals("DefaultPackage");
         importedClassesAndPackages = new ArrayList<String>();
+        if (!deferSetup) setUpImports(packagePath, links, isGroovy, className);
+        lastLineCol = new LineColumn(1, 1);
+    }
+
+    private void setUpImports(String packagePath, List<LinkArgument> links, boolean isGroovy, String className) {
         importedClassesAndPackages.add(packagePath + "/*");  // everything in this package
         if (isGroovy) {
             for (String pkg : ResolveVisitor.DEFAULT_IMPORTS) {
@@ -74,7 +82,6 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
         SimpleGroovyClassDoc currentClassDoc = new SimpleGroovyClassDoc(importedClassesAndPackages, className, links);
         currentClassDoc.setFullPathName(packagePath + FS + className);
         classDocs.put(currentClassDoc.getFullPathName(), currentClassDoc);
-        lastLineCol = new LineColumn(1, 1);
     }
 
     public Map<String, GroovyClassDoc> getGroovyClassDocs() {
@@ -120,6 +127,7 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
             current.setFullPathName(packagePath + FS + current.name());
             current.setTokenType(t.getType());
             processAnnotations(t, current);
+            processModifiers(t, current);
             classDocs.put(current.getFullPathName(), current);
             foundClasses.put(shortName, current);
             if (parent != null) {
@@ -129,6 +137,14 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
         }
     }
 
+    @Override
+    public void visitPackageDef(GroovySourceAST t, int visit) {
+        if (visit == OPENING_VISIT && deferSetup) {
+            String packageWithSlashes = extractImportPath(t);
+            setUpImports(packageWithSlashes, links, isGroovy, className);
+        }
+    }
+
     @Override
     public void visitImport(GroovySourceAST t, int visit) {
         if (visit == OPENING_VISIT) {
@@ -272,8 +288,7 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
     }
 
     private boolean isNested() {
-        GroovySourceAST gpn = getGrandParentNode();
-        return getParentNode() != null && gpn != null && (gpn.getType() == CLASS_DEF || gpn.getType() == INTERFACE_DEF);
+        return getCurrentClassDoc() != null;
     }
 
     private boolean isTopLevelConstruct(GroovySourceAST node) {
@@ -387,7 +402,7 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
         return grandParentNode != null && grandParentNode.getType() == LITERAL_new;
     }
 
-    private boolean processModifiers(GroovySourceAST t, SimpleGroovyMemberDoc memberDoc) {
+    private boolean processModifiers(GroovySourceAST t, SimpleGroovyAbstractableElementDoc memberOrClass) {
         GroovySourceAST modifiers = t.childOfType(MODIFIERS);
         if (modifiers != null) {
             AST currentModifier = modifiers.getFirstChild();
@@ -397,36 +412,43 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
                 int type = currentModifier.getType();
                 switch (type) {
                     case LITERAL_public:
-                        memberDoc.setPublic(true);
+                        memberOrClass.setPublic(true);
                         hasPublicVisibility = true;
                         break;
                     case LITERAL_protected:
-                        memberDoc.setProtected(true);
+                        memberOrClass.setProtected(true);
                         hasNonPublicVisibility = true;
                         break;
                     case LITERAL_private:
-                        memberDoc.setPrivate(true);
+                        memberOrClass.setPrivate(true);
                         hasNonPublicVisibility = true;
                         break;
                     case LITERAL_static:
-                        memberDoc.setStatic(true);
+                        memberOrClass.setStatic(true);
                         break;
                     case FINAL:
-                        memberDoc.setFinal(true);
+                        memberOrClass.setFinal(true);
                         break;
                     case ABSTRACT:
-                        memberDoc.setAbstract(true);
+                        memberOrClass.setAbstract(true);
                         break;
                 }
                 currentModifier = currentModifier.getNextSibling();
             }
-            if (!hasNonPublicVisibility && !(memberDoc instanceof GroovyFieldDoc)) {
-                // in groovy (and java asts turned into groovy by Groovifier), methods are assumed public, unless informed otherwise
-                memberDoc.setPublic(true);
+            if (!hasNonPublicVisibility && isGroovy && !(memberOrClass instanceof GroovyFieldDoc)) {
+                // in groovy methods and classes are assumed public, unless informed otherwise
+                memberOrClass.setPublic(true);
+            } else if (!hasNonPublicVisibility && !hasPublicVisibility && !isGroovy) {
+                memberOrClass.setPackagePrivate(true);
             }
-            if (memberDoc instanceof GroovyFieldDoc && !hasNonPublicVisibility && !hasPublicVisibility && isGroovy) return true;
+            if (memberOrClass instanceof GroovyFieldDoc && !hasNonPublicVisibility && !hasPublicVisibility && isGroovy) return true;
+        } else if (isGroovy && !(memberOrClass instanceof GroovyFieldDoc)) {
+            // in groovy methods and classes are assumed public, unless informed otherwise
+            memberOrClass.setPublic(true);
+        } else if (!isGroovy) {
+            memberOrClass.setPackagePrivate(true);
         }
-        return memberDoc instanceof GroovyFieldDoc && isGroovy;
+        return memberOrClass instanceof GroovyFieldDoc && isGroovy;
     }
 
     // todo - If no comment before node, then get comment from same node on parent class - ouch!
@@ -450,7 +472,7 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
     private boolean isMajorType(GroovySourceAST t) {
         if (t == null) return false;
         int tt = t.getType();
-        return tt == CLASS_DEF || tt == INTERFACE_DEF || tt == METHOD_DEF || tt == ANNOTATION_DEF ||
+        return tt == CLASS_DEF || tt == INTERFACE_DEF || tt == METHOD_DEF || tt == ANNOTATION_DEF || tt == ENUM_DEF ||
                 tt == VARIABLE_DEF || tt == ANNOTATION_FIELD_DEF || tt == ENUM_CONSTANT_DEF || tt == CTOR_IDENT;
     }
 
@@ -462,12 +484,13 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
         return returnValue;
     }
 
-    // preempt resolve as info is available here - TODO is this always safe?
+    // preempt resolve as info is partially available here (star imports won't match here)
     private String extractName(GroovySourceAST typeNode) {
         String typeName = typeNode.getText();
-        if (typeName.indexOf(".") == -1) {
+        if (typeName.indexOf("/") == -1) {
+            String slashName = "/" + typeName;
             for (String name : importedClassesAndPackages) {
-                if (name.endsWith(typeName)) {
+                if (name.endsWith(slashName)) {
                     typeName = name;
                 }
             }
@@ -616,11 +639,13 @@ public class SimpleGroovyClassDocAssembler extends VisitorAdapter implements Gro
     }
 
     private SimpleGroovyClassDoc getCurrentClassDoc() {
-        GroovySourceAST pn = getParentNode();
-        if (isTopLevelConstruct(pn)) return foundClasses.get(getIdentFor(pn));
-        GroovySourceAST gpn = getGrandParentNode();
-        if (isTopLevelConstruct(gpn)) return foundClasses.get(getIdentFor(gpn));
-        return null;
+        if (stack.isEmpty()) return null;
+        GroovySourceAST node = getParentNode();
+        if (isTopLevelConstruct(node)) return foundClasses.get(getIdentFor(node));
+        GroovySourceAST saved = stack.pop();
+        SimpleGroovyClassDoc result = getCurrentClassDoc();
+        stack.push(saved);
+        return result;
     }
 
     private String getIdentFor(GroovySourceAST gpn) {
