@@ -38,8 +38,9 @@ public class JavaStubGenerator
     private boolean java5 = false;
     private boolean requireSuperResolved = false;
     private File outputPath;
-    private List toCompile = new ArrayList();
-    private ArrayList propertyMethods = new ArrayList();
+    private List<String> toCompile = new ArrayList<String>();
+    private ArrayList<MethodNode> propertyMethods = new ArrayList<MethodNode>();
+    private ArrayList<ConstructorNode> constructors = new ArrayList<ConstructorNode>();
 
     public JavaStubGenerator(final File outputPath, final boolean requireSuperResolved, final boolean java5) {
         this.outputPath = outputPath;
@@ -51,14 +52,14 @@ public class JavaStubGenerator
     public JavaStubGenerator(final File outputPath) {
         this(outputPath, false, false);
     }
-    
+
     private void mkdirs(File parent, String relativeFile) {
         int index = relativeFile.lastIndexOf('/');
         if (index==-1) return;
         File dir = new File(parent,relativeFile.substring(0,index));
         dir.mkdirs();
     }
-    
+
     public void generateClass(ClassNode classNode) throws FileNotFoundException {
         // Only attempt to render our self if our super-class is resolved, else wait for it
         if (requireSuperResolved && !classNode.getSuperClass().isResolved()) {
@@ -79,7 +80,7 @@ public class JavaStubGenerator
         File file = new File(outputPath, fileName + ".java");
         FileOutputStream fos = new FileOutputStream(file);
         PrintWriter out = new PrintWriter(fos);
-        
+
         try {
             String packageName = classNode.getPackageName();
             if (packageName != null) {
@@ -105,6 +106,10 @@ public class JavaStubGenerator
     }
 
     private void genClassInner(ClassNode classNode, PrintWriter out) throws FileNotFoundException {
+    	if(classNode instanceof InnerClassNode && ((InnerClassNode) classNode).isAnonymous()) {
+    		// if it is an anonymous inner class, don't generate the stub code for it.
+    		return;
+    	}
         try {
             Verifier verifier = new Verifier() {
                 public void addCovariantMethods(ClassNode cn) {}
@@ -114,6 +119,25 @@ public class JavaStubGenerator
                     propertyMethods.add(method);
                 }
                 protected void addReturnIfNeeded(MethodNode node) {}
+                protected void addMethod(ClassNode node, boolean shouldBeSynthetic, String name, int modifiers, ClassNode returnType, Parameter[] parameters, ClassNode[] exceptions, Statement code) {
+                    propertyMethods.add(new MethodNode(name, modifiers, returnType, parameters, exceptions, code));
+                }
+                protected void addConstructor(Parameter[] newParams, ConstructorNode ctor, Statement code, ClassNode node) {
+                    constructors.add(new ConstructorNode(ctor.getModifiers(), newParams, ctor.getExceptions(), code));
+                }
+                protected void addDefaultParameters(DefaultArgsAction action, MethodNode method) {
+                    final Parameter[] parameters = method.getParameters();
+                    final Expression [] saved = new Expression[parameters.length];
+                    for (int i = 0; i < parameters.length; i++) {
+                        if (parameters[i].hasInitialExpression())
+                            saved[i] = parameters[i].getInitialExpression();
+                    }
+                    super.addDefaultParameters(action, method);
+                    for (int i = 0; i < parameters.length; i++) {
+                        if (saved[i] != null)
+                            parameters[i].setInitialExpression(saved[i]);
+                    }
+                }
             };
             verifier.visitClass(classNode);
 
@@ -164,13 +188,17 @@ public class JavaStubGenerator
             genMethods(classNode, out, isEnum);
 
             for (Iterator<InnerClassNode> inner = classNode.getInnerClasses(); inner.hasNext(); ) {
-                genClassInner(inner.next(), out);
+            	// GROOVY-4004: Clear the property methods from the outer class so that they don't get duplicated in inner ones
+            	propertyMethods.clear();
+                constructors.clear();
+            	genClassInner(inner.next(), out);
             }
 
             out.println("}");
         }
         finally {
             propertyMethods.clear();
+            constructors.clear();
         }
     }
 
@@ -187,7 +215,7 @@ public class JavaStubGenerator
                     String name = methodNode.getName();
                     Parameter[] params = methodNode.getParameters();
                     if (name.equals("values") && params.length==0) continue;
-                    if (name.equals("valueOf") && 
+                    if (name.equals("valueOf") &&
                         params.length==1 &&
                         params[0].getType().equals(ClassHelper.STRING_TYPE))
                     {
@@ -199,7 +227,8 @@ public class JavaStubGenerator
     }
 
     private void getConstructors(ClassNode classNode, PrintWriter out) {
-        List constrs = classNode.getDeclaredConstructors();
+        List<ConstructorNode> constrs = (List<ConstructorNode>) constructors.clone();
+        constrs.addAll(classNode.getDeclaredConstructors());
         if (constrs != null)
             for (Iterator it = constrs.iterator(); it.hasNext();) {
                 ConstructorNode constrNode = (ConstructorNode) it.next();
@@ -225,11 +254,11 @@ public class JavaStubGenerator
         genEnumFields(enumFields, out);
         for (Iterator iterator = normalFields.iterator(); iterator.hasNext();) {
             FieldNode fieldNode = (FieldNode) iterator.next();
-            genField(fieldNode, out);            
-        } 
+            genField(fieldNode, out);
+        }
     }
 
-    
+
     private void genEnumFields(List fields, PrintWriter out) {
         if (fields.size()==0) return;
         boolean first = true;
@@ -240,7 +269,7 @@ public class JavaStubGenerator
             } else {
                 first = false;
             }
-            out.print(fieldNode.getName());            
+            out.print(fieldNode.getName());
         }
         out.println(";");
     }
@@ -312,12 +341,12 @@ public class JavaStubGenerator
                 return c.getParameters();
             }
         }
-        
-        // fall back for parameterless constructor 
+
+        // fall back for parameterless constructor
         if (superType.isPrimaryClassNode()) {
             return Parameter.EMPTY_ARRAY;
         }
-        
+
         return null;
     }
 
@@ -335,7 +364,7 @@ public class JavaStubGenerator
                     out.print(", ");
                 }
             }
-            
+
             out.println(");");
             return;
         }
@@ -494,8 +523,8 @@ public class JavaStubGenerator
             out.print(type.redirect().getName().replace('$', '.'));
         }
     }
-    
-    
+
+
     private void writeGenericsBounds(PrintWriter out, ClassNode type, boolean skipName) {
         if (!skipName) printTypeName(type,out);
         if (!java5) return;
@@ -503,7 +532,7 @@ public class JavaStubGenerator
             writeGenericsBounds(out,type.getGenericsTypes());
         }
     }
-    
+
     private void writeGenericsBounds(PrintWriter out, GenericsType[] genericsTypes) {
         if (genericsTypes==null || genericsTypes.length==0) return;
         out.print('<');
@@ -513,7 +542,7 @@ public class JavaStubGenerator
         }
         out.print('>');
     }
-    
+
     private void writeGenericsBounds(PrintWriter out, GenericsType genericsType) {
         if (genericsType.isPlaceholder()) {
             out.print(genericsType.getName());
@@ -542,7 +571,7 @@ public class JavaStubGenerator
         if (parameters != null && parameters.length != 0) {
             for (int i = 0; i != parameters.length; ++i) {
                 printType(parameters[i].getType(), out);
-                
+
                 out.print(" ");
                 out.print(parameters[i].getName());
 
@@ -551,7 +580,7 @@ public class JavaStubGenerator
                 }
             }
         }
-        
+
         out.print(")");
     }
 
@@ -564,7 +593,7 @@ public class JavaStubGenerator
 
         if ((modifiers & Opcodes.ACC_PRIVATE) != 0)
             out.print("private ");
-        
+
         if ((modifiers & Opcodes.ACC_STATIC) != 0)
             out.print("static ");
 
@@ -582,7 +611,7 @@ public class JavaStubGenerator
         // HACK: Add the default imports... since things like Closure and GroovyObject seem to parse out w/o fully qualified classnames.
         //
         imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
-        
+
         ModuleNode moduleNode = classNode.getModule();
         for (ImportNode importNode : moduleNode.getStarImports()) {
             imports.add(importNode.getPackageName());
