@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 the original author or authors.
+ * Copyright 2008-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,10 +13,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.codehaus.groovy.transform;
 
+import groovy.lang.GroovyObject;
 import groovy.lang.Immutable;
+import groovy.lang.MetaClass;
+import groovy.lang.MissingPropertyException;
 import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.*;
@@ -59,6 +61,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
             java.math.BigInteger.class,
             java.math.BigDecimal.class,
             java.awt.Color.class,
+            java.net.URI.class,
     };
     private static final Class MY_CLASS = Immutable.class;
     private static final ClassNode MY_TYPE = new ClassNode(MY_CLASS);
@@ -75,7 +78,6 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     private static final ClassNode SELF_TYPE = new ClassNode(ImmutableASTTransformation.class);
     private static final Token COMPARE_EQUAL = Token.newSymbol(Types.COMPARE_EQUAL, -1, -1);
     private static final Token COMPARE_NOT_EQUAL = Token.newSymbol(Types.COMPARE_NOT_EQUAL, -1, -1);
-    private static final Token COMPARE_IDENTICAL = Token.newSymbol(Types.COMPARE_IDENTICAL, -1, -1);
     private static final Token ASSIGN = Token.newSymbol(Types.ASSIGN, -1, -1);
 
     public void visit(ASTNode[] nodes, SourceUnit source) {
@@ -98,12 +100,12 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
                 cNode.setModifiers(cNode.getModifiers() | ACC_FINAL);
             }
 
-            final List<PropertyNode> pList = cNode.getProperties();
+            final List<PropertyNode> pList = getInstanceProperties(cNode);
             for (PropertyNode pNode : pList) {
                 adjustPropertyForImmutability(pNode, newNodes);
             }
             for (PropertyNode pNode : newNodes) {
-                pList.remove(pNode);
+                cNode.getProperties().remove(pNode);
                 addProperty(cNode, pNode);
             }
             final List<FieldNode> fList = cNode.getFields();
@@ -117,6 +119,17 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         }
     }
 
+    private boolean hasDeclaredMethod(ClassNode cNode, String name, int argsCount) {
+        List<MethodNode> ms = cNode.getDeclaredMethods(name);
+        for(MethodNode m : ms) {
+           Parameter[] paras = m.getParameters();
+           if(paras != null && paras.length == argsCount) {
+                return true;
+           }
+        }
+        return false;
+    }
+
     private void ensureNotPublic(String cNode, FieldNode fNode) {
         String fName = fNode.getName();
         // TODO: do we need to lock down things like: $ownClass
@@ -126,10 +139,14 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private void createHashCode(ClassNode cNode) {
+        // make a public method if none exists otherwise try a private method with leading underscore
+        boolean hasExistingHashCode = hasDeclaredMethod(cNode, "hashCode", 0);
+        if (hasExistingHashCode && hasDeclaredMethod(cNode, "_hashCode", 0)) return;
+
         final FieldNode hashField = cNode.addField("$hash$code", ACC_PRIVATE | ACC_SYNTHETIC, ClassHelper.int_TYPE, null);
         final BlockStatement body = new BlockStatement();
         final Expression hash = new FieldExpression(hashField);
-        final List<PropertyNode> list = cNode.getProperties();
+        final List<PropertyNode> list = getInstanceProperties(cNode);
 
         body.addStatement(new IfStatement(
                 isZeroExpr(hash),
@@ -139,12 +156,17 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
 
         body.addStatement(new ReturnStatement(hash));
 
-        cNode.addMethod(new MethodNode("hashCode", ACC_PUBLIC, ClassHelper.int_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, body));
+        cNode.addMethod(new MethodNode(hasExistingHashCode ? "_hashCode" : "hashCode", hasExistingHashCode ? ACC_PRIVATE : ACC_PUBLIC,
+                ClassHelper.int_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, body));
     }
 
     private void createToString(ClassNode cNode) {
+        // make a public method if none exists otherwise try a private method with leading underscore
+        boolean hasExistingToString = hasDeclaredMethod(cNode, "toString", 0);
+        if (hasExistingToString && hasDeclaredMethod(cNode, "_toString", 0)) return;
+        
         final BlockStatement body = new BlockStatement();
-        final List<PropertyNode> list = cNode.getProperties();
+        final List<PropertyNode> list = getInstanceProperties(cNode);
         // def _result = new StringBuffer()
         final Expression result = new VariableExpression("_result");
         final Expression init = new ConstructorCallExpression(STRINGBUFFER_TYPE, MethodCallExpression.NO_ARGUMENTS);
@@ -169,7 +191,8 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         }
         body.addStatement(append(result, new ConstantExpression(")")));
         body.addStatement(new ReturnStatement(new MethodCallExpression(result, "toString", MethodCallExpression.NO_ARGUMENTS)));
-        cNode.addMethod(new MethodNode("toString", ACC_PUBLIC, ClassHelper.STRING_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, body));
+        cNode.addMethod(new MethodNode(hasExistingToString ? "_toString" : "toString", hasExistingToString ? ACC_PRIVATE : ACC_PUBLIC,
+                ClassHelper.STRING_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, body));
     }
 
     private Statement toStringPropertyName(Expression result, String fName) {
@@ -204,6 +227,10 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private void createEquals(ClassNode cNode) {
+        // make a public method if none exists otherwise try a private method with leading underscore
+        boolean hasExistingEquals = hasDeclaredMethod(cNode, "equals", 1);
+        if (hasExistingEquals && hasDeclaredMethod(cNode, "_equals", 1)) return;
+
         final BlockStatement body = new BlockStatement();
         Expression other = new VariableExpression("other");
 
@@ -212,7 +239,9 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         body.addStatement(returnFalseIfWrongType(cNode, other));
         body.addStatement(returnTrueIfIdentical(VariableExpression.THIS_EXPRESSION, other));
 
-        final List<PropertyNode> list = cNode.getProperties();
+        body.addStatement(new ExpressionStatement(new BinaryExpression(other, ASSIGN, new CastExpression(cNode, other))));
+
+        final List<PropertyNode> list = getInstanceProperties(cNode);
         // fields
         for (PropertyNode pNode : list) {
             body.addStatement(returnFalseIfPropertyNotEqual(pNode, other));
@@ -222,7 +251,8 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         body.addStatement(new ReturnStatement(ConstantExpression.TRUE));
 
         Parameter[] params = {new Parameter(OBJECT_TYPE, "other")};
-        cNode.addMethod(new MethodNode("equals", ACC_PUBLIC, ClassHelper.boolean_TYPE, params, ClassNode.EMPTY_ARRAY, body));
+        cNode.addMethod(new MethodNode(hasExistingEquals ? "_equals" : "equals", hasExistingEquals ? ACC_PRIVATE : ACC_PUBLIC,
+                ClassHelper.boolean_TYPE, params, ClassNode.EMPTY_ARRAY, body));
     }
 
     private Statement returnFalseIfWrongType(ClassNode cNode, Expression other) {
@@ -276,7 +306,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
             throw new RuntimeException("Explicit constructors not allowed for " + MY_TYPE_NAME + " class: " + cNode.getNameWithoutPackage());
         }
 
-        List<PropertyNode> list = cNode.getProperties();
+        List<PropertyNode> list = getInstanceProperties(cNode);
         boolean specialHashMapCase = list.size() == 1 && list.get(0).getField().getType().equals(HASHMAP_TYPE);
         if (specialHashMapCase) {
             createConstructorMapSpecial(cNode, constructorStyle, list);
@@ -286,6 +316,16 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         }
     }
 
+    private List<PropertyNode> getInstanceProperties(ClassNode cNode) {
+        final List<PropertyNode> result = new ArrayList<PropertyNode>();
+        for (PropertyNode pNode : cNode.getProperties()) {
+            if (!pNode.isStatic()) {
+                result.add(pNode);
+            }
+        }
+        return result;
+    }
+
     private void createConstructorMapSpecial(ClassNode cNode, FieldExpression constructorStyle, List<PropertyNode> list) {
         final BlockStatement body = new BlockStatement();
         body.addStatement(createConstructorStatementMapSpecial(list.get(0).getField()));
@@ -297,6 +337,9 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         for (PropertyNode pNode : list) {
             body.addStatement(createConstructorStatement(cNode, pNode));
         }
+        // check for missing properties
+        Expression checkArgs = new ArgumentListExpression(new VariableExpression("this"), new VariableExpression("args"));
+        body.addStatement(new ExpressionStatement(new StaticMethodCallExpression(SELF_TYPE, "checkPropNames", checkArgs)));
         createConstructorMapCommon(cNode, constructorStyle, body);
     }
 
@@ -490,11 +533,11 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private BooleanExpression isZeroExpr(Expression expr) {
-        return new BooleanExpression(new BinaryExpression(expr, COMPARE_EQUAL, new ConstantExpression(Integer.valueOf(0))));
+        return new BooleanExpression(new BinaryExpression(expr, COMPARE_EQUAL, new ConstantExpression(0)));
     }
 
     private BooleanExpression isOneExpr(Expression expr) {
-        return new BooleanExpression(new BinaryExpression(expr, COMPARE_EQUAL, new ConstantExpression(Integer.valueOf(1))));
+        return new BooleanExpression(new BinaryExpression(expr, COMPARE_EQUAL, new ConstantExpression(1)));
     }
 
     private BooleanExpression notEqualsExpr(PropertyNode pNode, Expression other) {
@@ -504,7 +547,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private BooleanExpression identicalExpr(Expression self, Expression other) {
-        return new BooleanExpression(new BinaryExpression(self, COMPARE_IDENTICAL, other));
+        return new BooleanExpression(new MethodCallExpression(self, "is", new ArgumentListExpression(other)));
     }
 
     private BooleanExpression notEqualClasses(ClassNode cNode, Expression other) {
@@ -549,11 +592,12 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private static String createErrorMessage(String className, String fieldName, String typeName, String mode) {
-        return MY_TYPE_NAME + " processor doesn't know how to handle field '" + fieldName + "' of type '" + prettyTypeName(typeName) + "' while " + mode + " class " + className + ".\n" +
+        return MY_TYPE_NAME + " processor doesn't know how to handle field '" + fieldName + "' of type '" +
+                prettyTypeName(typeName) + "' while " + mode + " class " + className + ".\n" +
                 MY_TYPE_NAME + " classes currently only support properties with known immutable types " +
                 "or types where special handling achieves immutable behavior, including:\n" +
-                "- Strings, primitive types, wrapper types, BigInteger and BigDecimal\n" +
-                "- enums, other " + MY_TYPE_NAME + " classes and known immutables (java.awt.Color)\n" +
+                "- Strings, primitive types, wrapper types, BigInteger and BigDecimal, enums\n" +
+                "- other " + MY_TYPE_NAME + " classes and known immutables (java.awt.Color, java.net.URI)\n" +
                 "- Cloneable classes, collections, maps and arrays, and other classes with special handling (java.util.Date)\n" +
                 "Other restrictions apply, please see the groovydoc for " + MY_TYPE_NAME + " for further details";
     }
@@ -597,4 +641,11 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         throw new RuntimeException(createErrorMessage(className, fieldName, typeName, "constructing"));
     }
 
+    public static void checkPropNames(GroovyObject instance, Map<String, Object> args) {
+        final MetaClass metaClass = instance.getMetaClass();
+        for (String k : args.keySet()) {
+            if (metaClass.hasProperty(instance, k) == null)
+                throw new MissingPropertyException(k, instance.getClass());
+        }
+    }
 }
