@@ -19,7 +19,6 @@ package org.codehaus.groovy.vmplugin.v5;
 import java.lang.reflect.*;
 import java.lang.annotation.*;
 import java.util.List;
-import java.util.Iterator;
 
 import org.codehaus.groovy.GroovyBugError;
 import org.codehaus.groovy.vmplugin.VMPlugin;
@@ -113,6 +112,7 @@ public class Java5 implements VMPlugin {
 
     private ClassNode configureWildcardType(WildcardType wildcardType) {
         ClassNode base = ClassHelper.makeWithoutCaching("?");
+        base.setRedirect(ClassHelper.OBJECT_TYPE);
         //TODO: more than one lower bound for wildcards?
         ClassNode[] lowers = configureTypes(wildcardType.getLowerBounds());
         ClassNode lower = null;
@@ -140,6 +140,7 @@ public class Java5 implements VMPlugin {
         ClassNode cn = ClassHelper.makeWithoutCaching(tv.getName());
         cn.setGenericsPlaceHolder(true);
         ClassNode cn2 = ClassHelper.makeWithoutCaching(tv.getName());
+        cn2.setGenericsPlaceHolder(true);
         GenericsType[] gts = new GenericsType[]{new GenericsType(cn2)};
         cn.setGenericsTypes(gts);
         cn.setRedirect(ClassHelper.OBJECT_TYPE);
@@ -150,7 +151,13 @@ public class Java5 implements VMPlugin {
         if (ta.length == 0) return null;
         GenericsType[] gts = new GenericsType[ta.length];
         for (int i = 0; i < ta.length; i++) {
-            gts[i] = new GenericsType(configureType(ta[i]));
+            ClassNode t = configureType(ta[i]);
+            if (ta[i] instanceof WildcardType) {
+                GenericsType[] gen = t.getGenericsTypes();
+                gts[i] = gen[0];
+            } else {
+                gts[i] = new GenericsType(t);
+            }
         }
         return gts;
     }
@@ -159,21 +166,11 @@ public class Java5 implements VMPlugin {
         return PLUGIN_DGM;
     }
 
-    private void setAnnotationMetaData(ClassNode cn) {
-        Annotation[] annotations = cn.getTypeClass().getAnnotations();
+    private void setAnnotationMetaData(Annotation[] annotations, AnnotatedNode an) {
         for (Annotation annotation : annotations) {
             AnnotationNode node = new AnnotationNode(ClassHelper.make(annotation.annotationType()));
             configureAnnotation(node, annotation);
-            cn.addAnnotation(node);
-        }
-    }
-
-    private void setAnnotationMetaData(MethodNode mn, Method m) {
-        Annotation[] annotations = m.getAnnotations();
-        for (Annotation annotation : annotations) {
-            AnnotationNode node = new AnnotationNode(ClassHelper.make(annotation.annotationType()));
-            configureAnnotation(node, annotation);
-            mn.addAnnotation(node);
+            an.addAnnotation(node);
         }
     }
 
@@ -193,9 +190,8 @@ public class Java5 implements VMPlugin {
             if (!(exp instanceof ListExpression)) return;
             ListExpression le = (ListExpression) exp;
             int bitmap = 0;
-
-            for (Iterator it=le.getExpressions().iterator(); it.hasNext();) {
-                PropertyExpression element = (PropertyExpression) it.next();
+            for (Expression e : le.getExpressions()) {
+                PropertyExpression element = (PropertyExpression) e;
                 String name = element.getPropertyAsString();
                 ElementType value = ElementType.valueOf(name);
                 bitmap |= getElementCode(value);
@@ -210,7 +206,6 @@ public class Java5 implements VMPlugin {
         for (AnnotationNode an : annotations) {
             configureAnnotationFromDefinition(an, node);
         }
-
         configureAnnotationFromDefinition(node, node);
     }
 
@@ -226,12 +221,46 @@ public class Java5 implements VMPlugin {
         } else if (type == Target.class) {
             Target t = (Target) annotation;
             ElementType[] elements = t.value();
-            int bitmap = 0;
+            ListExpression elementExprs = new ListExpression();
             for (ElementType element : elements) {
-                bitmap |= getElementCode(element);
+                elementExprs.addExpression(new PropertyExpression(
+                        new ClassExpression(ClassHelper.ELEMENT_TYPE_TYPE), element.name()));
             }
-            node.setAllowedTargets(bitmap);
+            node.setMember("value", elementExprs);
         }
+        else {
+            Method[] declaredMethods = type.getDeclaredMethods();
+            for (int i = 0; i < declaredMethods.length; i++) {
+                Method declaredMethod = declaredMethods[i];
+                try {
+                    Object value = declaredMethod.invoke(annotation);
+                    Expression valueExpression = annotationValueToExpression(value);
+                    if (valueExpression == null)
+                        continue;
+                    node.setMember(declaredMethod.getName(), valueExpression);
+                } catch (IllegalAccessException e) {
+                } catch (InvocationTargetException e) {
+                }
+            }
+        }
+    }
+
+    private Expression annotationValueToExpression (Object value) {
+        if (value == null || value instanceof String || value instanceof Number || value instanceof Character || value instanceof Boolean)
+            return new ConstantExpression(value);
+
+        if (value instanceof Class)
+            return new ClassExpression(ClassHelper.makeWithoutCaching((Class)value));
+
+        if (value.getClass().isArray()) {
+            ListExpression elementExprs = new ListExpression();
+            int len = Array.getLength(value);
+            for (int i = 0; i != len; ++i)
+                elementExprs.addExpression(annotationValueToExpression(Array.get(value, i)));
+            return elementExprs;
+        }
+
+        return null;
     }
 
     private void setRetentionPolicy(RetentionPolicy value, AnnotationNode node) {
@@ -293,7 +322,8 @@ public class Java5 implements VMPlugin {
             ClassNode[] exceptions = makeClassNodes(compileUnit, m.getGenericExceptionTypes(), m.getExceptionTypes());
             MethodNode mn = new MethodNode(m.getName(), m.getModifiers(), ret, params, exceptions, null);
             setMethodDefaultValue(mn, m);
-            setAnnotationMetaData(mn, m);
+            setAnnotationMetaData(m.getAnnotations(), mn);
+            mn.setGenericsTypes(configureTypeVariable(m.getTypeParameters()));
             classNode.addMethod(mn);
         }
         Constructor[] constructors = clazz.getDeclaredConstructors();
@@ -306,8 +336,12 @@ public class Java5 implements VMPlugin {
         Class sc = clazz.getSuperclass();
         if (sc != null) classNode.setUnresolvedSuperClass(makeClassNode(compileUnit, clazz.getGenericSuperclass(), sc));
         makeInterfaceTypes(compileUnit, classNode, clazz);
-        setAnnotationMetaData(classNode);
+        setAnnotationMetaData(classNode.getTypeClass().getAnnotations(), classNode);
 
+        PackageNode packageNode = classNode.getPackage();
+        if (packageNode != null) {
+            setAnnotationMetaData(classNode.getTypeClass().getPackage().getAnnotations(), packageNode);
+        }
     }
 
     private void makeInterfaceTypes(CompileUnit cu, ClassNode classNode, Class clazz) {
