@@ -1,5 +1,5 @@
 /*
- * $Id: GroovyShellTest.java 17820 2009-09-30 22:37:55Z blackdrag $version
+ * $Id: GroovyShellTest.java 18772 2009-12-29 08:39:23Z paulk $version
  * Nov 23, 2003 9:02:55 PM $user Exp $
  * 
  * Copyright 2003 (C) Sam Pullara. All Rights Reserved.
@@ -39,13 +39,19 @@ import junit.framework.Test;
 import junit.framework.TestSuite;
 import junit.textui.TestRunner;
 import org.codehaus.groovy.control.CompilerConfiguration;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.PrintStream;
+import java.io.File;
+import java.net.URL;
 import java.util.HashMap;
 import java.util.Map;
 
 /**
  * @author sam
+ * @author Merlyn Albery-Speyer
  */
 public class GroovyShellTest extends GroovyTestCase {
 
@@ -119,4 +125,40 @@ public class GroovyShellTest extends GroovyTestCase {
         shell.evaluate(script);
      
     }
+
+    public void testWithGCSWithURL() throws Exception {
+    	String scriptFileName = "src/test/groovy/bugs/GROOVY3934Helper.groovy";
+    	File helperScript = new File(scriptFileName);
+    	if(!helperScript.exists()) {
+    		fail("File " + scriptFileName + " does not exist");
+    	} else {
+        	URL url = helperScript.toURL();
+        	GroovyCodeSource gcs = new GroovyCodeSource(url);
+        	GroovyShell shell = new GroovyShell();
+            Object result = shell.evaluate(gcs);
+            assertEquals("GROOVY3934Helper script called", result);
+    	}
+    }
+    
+    public void testLaunchesJUnitTestSuite() throws Exception {
+    	// create a valid (empty) test suite on disk
+    	String testName = "GroovyShellTestJUnit3Test"+System.currentTimeMillis();
+		File testSuite = new File(System.getProperty("java.io.tmpdir"), testName);
+		DefaultGroovyMethods.write(testSuite, "import junit.framework.*; \r\n"+
+			"public class "+testName+" extends TestSuite { \r\n"+
+			"    public static Test suite() { \r\n"+
+			"        return new TestSuite(); \r\n"+
+			"    } \r\n" +
+			"} \r\n");
+		testSuite.deleteOnExit();
+		
+		PrintStream out = System.out;
+		System.setOut( new PrintStream(new ByteArrayOutputStream()) );
+		try {
+		    // makes this more of an integration test than a unit test...
+		    GroovyShell.main( new String[] { testSuite.getCanonicalPath() });
+		} finally {
+		    System.setOut( out );
+		}
+	} 
 }
