@@ -1,5 +1,5 @@
 /*
- $Id: DefaultGroovyMethodsTest.java 17058 2009-07-26 00:18:28Z paulk $
+ $Id: DefaultGroovyMethodsTest.java 19499 2010-03-06 11:46:11Z paulk $
 
  Copyright 2003 (C) James Strachan and Bob Mcwhirter. All Rights Reserved.
 
@@ -49,11 +49,16 @@ package org.codehaus.groovy.runtime;
 import groovy.lang.Closure;
 import groovy.util.GroovyTestCase;
 
+import java.io.BufferedReader;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.net.URI;
 import java.net.URL;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -205,6 +210,34 @@ public class DefaultGroovyMethodsTest extends GroovyTestCase {
         assertFalse(DefaultGroovyMethods.isNumber(nonNumberStr));
     }
 
+    public void testGetBytes() {
+        byte[] bytes = {42,45,47,14,10,84};
+        ByteArrayInputStream is = new ByteArrayInputStream(bytes);
+        try {
+            byte[] answer = DefaultGroovyMethods.getBytes(is);
+            assertEquals(bytes.length, answer.length);
+            for (int i = 0; i < bytes.length; i++) {
+                assertEquals(bytes[i], answer[i]);       
+            }
+        } catch (IOException e) {
+            fail();
+        }
+    }
+
+    public void testSetBytes() {
+        byte[] bytes = {42,45,47,14,10,84};
+        ByteArrayOutputStream os = new ByteArrayOutputStream();
+        try {
+            DefaultGroovyMethods.setBytes(os, bytes);
+            byte[] answer = os.toByteArray();
+            assertEquals(bytes.length, answer.length);
+            for (int i = 0; i < bytes.length; i++) {
+                assertEquals(bytes[i], answer[i]);
+            }
+        } catch (IOException e) {
+            fail();
+        }
+    }
 
     public void testDownto() {
         final int[] count = new int[]{0};
@@ -223,4 +256,16 @@ public class DefaultGroovyMethodsTest extends GroovyTestCase {
         DefaultGroovyMethods.downto(new BigInteger("1"), new BigDecimal("0.123"), closure);
         assertEquals(count[0], 1);
     }
+
+    public void testBulkCollectionForArrayOperations() {
+        List<String> list = new ArrayList<String>();
+        assertTrue(DefaultGroovyMethods.addAll(list, "abcd".split("")));
+        assertTrue(DefaultGroovyMethods.removeAll(list, "def".split("")));
+        assertTrue(DefaultGroovyMethods.retainAll(list, "bcd".split("")));
+        List<String> bAndC = Arrays.asList("b", "c");
+        assertTrue(DefaultGroovyMethods.containsAll(list, bAndC.toArray()));
+        assertEquals(list, bAndC);
+        assertTrue(DefaultGroovyMethods.addAll(list, 1, Arrays.asList("a", "s", "i").toArray(new String[3])));
+        assertEquals(list, Arrays.asList("b", "a", "s", "i", "c"));
+    }
 }
